<?php

namespace Teknisa\DF\Modules\Saida\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Utilities;

class Dfs65001ParametrosNfseInscmuniController extends \Teknisa\DF\Controllers\TeknisaCrud {
    protected $dataSourceName = 'paramnfseinscmuni_municipio';
    protected $dataSourceNameToSave = 'paramnfseinscmuni';
    protected $paramService;

    public function __construct($crudService, $parameterBag, $parametrosNfseInscmuniService) {
        parent::__construct($crudService, $parameterBag);
        $this->paramService = $parametrosNfseInscmuniService;
        date_default_timezone_set("America/Sao_Paulo");
    }

    public function save(Request $request, Response $response) {
    	try {
        	$dados = $request->getDataSet()->getRows();
        	$dados = $dados[0];
        	$existe = $this->paramService->verificarInscricaoMunicipal($dados);
        	
        	$dados["__columns_to_return"] = array("NRPARAMNFSEINSCMUNI", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO");
        	
    	    if ($existe == true) {
    	        $response->setError(new Response\Error("Inscrição Municipal já informada.", 500));
    	    }
    	    else {
    	        $retorno = array();
            	if (!empty($dados["CDSENHACERTDIG"]) && $dados["CDSENHACERTDIG"] != $dados["CDSENHACERTDIG_OLD"]) {
        	    	$senha = $dados["CDSENHACERTDIG"];
        	    	$senha = $this->encrypt($senha, 'teknisa');
        	    	$dados["CDSENHACERTDIG"] = $senha;
        	    	$retorno["CDSENHACERTDIG"] = $senha;
        	    }
        	    if (!empty($dados["CDSENHAACESSOWS"]) && $dados["CDSENHAACESSOWS"] != $dados["CDSENHAACESSOWS_OLD"]) {
        	    	$senha = $dados["CDSENHAACESSOWS"];
        	    	$senha = $this->encrypt($senha, 'teknisa');
        	    	$dados["CDSENHAACESSOWS"] = $senha;
        	    	$retorno["CDSENHAACESSOWS"] = $senha;
                }
                
                if (!empty($dados["CDFRASESECRETA"])) {
        	    	$cdfrasesecreta = $dados["CDFRASESECRETA"];
        	    	$cdfrasesecreta = $this->encrypt($cdfrasesecreta, 'teknisa');
        	    	$dados["CDFRASESECRETA"] = $cdfrasesecreta;
        	    	$retorno["CDFRASESECRETA"] = $cdfrasesecreta;
        	    }
        	    
        	    $dataset = new DataSet('paramnfseinscmuni', array($dados));
                $requestSalvar = new DTO\Request\DataSet($dataset, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                
                parent::save($requestSalvar, $response);

        	    $columnsToReturn = $this->crudService->getColumnsToReturn();

                if (isset($retorno["CDSENHACERTDIG"])) {
                    $columnsToReturn[0]["CDSENHACERTDIG"] = $retorno["CDSENHACERTDIG"];
                }
                if (isset($retorno["CDSENHAACESSOWS"])) {
                    $columnsToReturn[0]["CDSENHAACESSOWS"] = $retorno["CDSENHAACESSOWS"];
                }
                
        	    $response->addDataSet(new DataSource\DataSet("columnsToReturn", $columnsToReturn));
    	    }
    	}
    	catch (\Exception $e) {
    	    $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
    	}
    }
    
    public function remove(Request $request, Response $response){
        try{
            $row = $request->getRow();
            $this->paramService->remove($row);
        } catch (\Exception $e) {
    	    $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
    	}
    }
    
    private function encrypt($text, $salt = 'teknisa') {
       return Utilities::zeedhiEncrypt($text, $salt);
    }
    
    public function decrypt($text, $salt = 'teknisa'){
       return Utilities::zeedhiDecrypt($text, $salt);
    }
}