<?php

namespace Teknisa\DF\Modules\Saida\Services;

use Teknisa\DF\Modules\Saida\Factories\Dfs74000CalculoReceitaBrutaSimplesNacionalFactory;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Model\Fiscal\Entities\Mapaapuracsn;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DataSource\DataSet;


class Dfs74000CalculoReceitaBrutaSimplesNacionalService {
    protected $entityManager;
    protected $environment;
    protected $nrorg;
    protected $factoryCalculoReceitaBruta;
    protected $dataSourceManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Environment $environment, Dfs74000CalculoReceitaBrutaSimplesNacionalFactory $factoryCalculoReceitaBruta, $dataSourceManager) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->nrorg = $environment->getNrorgTrab();
        $this->factoryCalculoReceitaBruta = $factoryCalculoReceitaBruta;
        $this->dataSourceManager = $dataSourceManager;
    }


    public function validacoesSave($row){

        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $data = $this->formataData($row['DTREFRECEITABR']);

            $mesPosterior = $this->factoryCalculoReceitaBruta->buscaMesPosterior($this->nrorg,$row['CDEMPRESA'], $row['CDSIMPLESNACIONAL'], $data->format('d/m/Y') );
            $mesAtual = $this->factoryCalculoReceitaBruta->buscaMesAtual($this->nrorg,$row['CDEMPRESA'], $row['CDSIMPLESNACIONAL'], $data->format('d/m/Y') );

            $array = array('mesAtual' =>$mesAtual ,'mesPosterior' =>$mesPosterior );
            $connection->commit();
            return $array;

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function buscaNotasFiscaisMes($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $notasFiscaisMes = $this->factoryCalculoReceitaBruta->buscaNotasFiscaisMes($row['CDEMPRESA'], $row['CDSIMPLESNACIONAL'], $row['DTREFRECEITABR']);

            return $notasFiscaisMes;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function buscaMesPosterior($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $data = $this->formataData($row['DTREFRECEITABR']);
            $mesPosterior = $this->factoryCalculoReceitaBruta->buscaMesPosterior($this->nrorg,$row['CDEMPRESA'], $row['CDSIMPLESNACIONAL'], $data->format('d/m/Y') );

            $connection->commit();
            return $mesPosterior;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function realizaCalculoMes($mesAtual,$statusMes){
        try{
            $valorTotal    = 0;
            $saldoNegativo = 0;

            $valorCompoem = $this->factoryCalculoReceitaBruta->calculaReceitaBrutaMes($mesAtual['CDEMPRESA'],'C',$mesAtual['CDSIMPLESNACIONAL'],$mesAtual['DTREFRECEITABR']);
            $valorCompoem = $valorCompoem[0]['VRNOTAFISC'];

            $valorDeduz   = $this->factoryCalculoReceitaBruta->calculaReceitaBrutaMes($mesAtual['CDEMPRESA'],'D',$mesAtual['CDSIMPLESNACIONAL'],$mesAtual['DTREFRECEITABR']);
            $valorDeduz   = $valorDeduz[0]['VRNOTAFISC'];

            $saldoNegativoMesAnterior = $this->factoryCalculoReceitaBruta->buscaSaldoNegativoMesAnterior($mesAtual['CDEMPRESA'],$mesAtual['CDSIMPLESNACIONAL'],$mesAtual['DTREFRECEITABR']);
            if(empty($saldoNegativoMesAnterior)){
                $saldoNegativoMesAnterior = 0.0;
            }else{
                $saldoNegativoMesAnterior = $saldoNegativoMesAnterior[0]['VRSALDONEGMES'];
            }

            $valorTotal = $valorCompoem - ( $valorDeduz + $saldoNegativoMesAnterior ) ;
            if($valorTotal<0){
                $saldoNegativo = $valorTotal * -1;
                $valorTotal = 0;
            }

            $this->factoryCalculoReceitaBruta->atualizaReceitaBruta($statusMes,$valorTotal,$valorCompoem,$saldoNegativo,$valorDeduz,'C',$mesAtual['CDEMPRESA'],$mesAtual['CDSIMPLESNACIONAL'],$mesAtual['DTREFRECEITABR'], $this->nrorg);
            $this->entityManager->flush();

            $this->atualizaNotasFiscas($mesAtual);
            $array = [
                "valorCompoem"  => $valorCompoem,
                "valorDeduz"    => $valorDeduz,
                "valorTotal"    => $valorTotal,
                "saldoNegativo" => $saldoNegativo,
                "idstatusmes"   => $statusMes,
                "idinfmanual"   => 'C'
            ];
            return $array;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteMesReceita($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $mesAtual     = $row['mesAtual'];
            $mesPosterior = $row['mesPosterior'];

            $array[] = $mesAtual;
            $dataSet = new DataSet('receitabruta_completa', $array);
            $this->dataSourceManager->delete($dataSet);

            $this->atualizaMesesPosteriores($mesPosterior,'C');

            $connection->commit();

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function saveMesReceitaBruta($mesAtual,$mesPosterior){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            if($mesAtual[0]['__is_new']){
                $data = DateUtil::getDataDeString($mesAtual[0]['DTREFRECEITABR']);
                $mesAtual[0]['DTREFRECEITABR']= $data->format("d/m/Y H:i:s");
            }
            $dataSet = new DataSet('receitabruta', $mesAtual);
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();

            if(!empty($mesPosterior)){
                $this->atualizaMesesPosteriores($mesPosterior,'C');
            }

            $connection->commit();

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function calculaReceitaBrutaMes($row){
        try{

            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $mesAtual     = $row['mesAtual'];
            $mesPosterior = $row['mesPosterior'];

            $valoresMes = $this->realizaCalculoMes($mesAtual,'C');
            //atualiza o calculo da receita de todos os meses posteriores
            if(!empty($mesPosterior)){
                $this->atualizaMesesPosteriores($mesPosterior,'C');
            }
            $connection->commit();
            return $valoresMes;


        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function bloqueaMesReceita($row){
        try{

            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $mesAtual     = $row;
            $mesPosterior = $this->buscaMesPosterior($row);

            $valoresMes = $this->realizaCalculoMes($mesAtual,'B');
            //atualiza o calculo da receita de todos os meses posteriores
            if(!empty($mesPosterior)){
                $this->atualizaMesesPosteriores($mesPosterior,'C');
            }
            $connection->commit();
            return $valoresMes;


        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function atualizaMesesPosteriores($meses,$statusMes){
        try{
            foreach ($meses as &$mes) {
                $valoresMes = $this->realizaCalculoMes($mes,$statusMes);
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function atualizaNotasFiscas($mesAtual){
        $notasFiscaisMes = $this->factoryCalculoReceitaBruta->buscaNotasFiscaisMes($mesAtual['CDEMPRESA'], $mesAtual['CDSIMPLESNACIONAL'], $mesAtual['DTREFRECEITABR']);
        //caso exista notas fiscais atualiza a data do calculo da teceita
        if(! empty($notasFiscaisMes)){
            foreach ($notasFiscaisMes as &$nota) {
                $this->factoryCalculoReceitaBruta->atualizaNotaFiscal($mesAtual['DTREFRECEITABR'], $nota['CDFILIAL'], $nota['NRLANCTONF']);
                $this->entityManager->flush();
            }
        }
    }

    public static function formataData($data){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

}