<?php

namespace Teknisa\DF\Services;

use Fiscal\Entities\Dadosadic;
use Fiscal\Entities\Notafiscal;
use Geral\Entities\Formpgto;
use Geral\Entities\Itemproduto;
use Geral\Entities\Paramgeral;
use Geral\Entities\Tipooperac;
use Indeterminado\Entities\Adicdecimpnfe;
use Indeterminado\Entities\Dadosfisco;
use Indeterminado\Entities\Decimpnfe;
use Teknisa\DF\Factories\TransmissaoNfeFactory;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\NewCode;
use Teknisa\DF\Services\NFe\NFe;
use Teknisa\DF\Services\NFe\Make;
use Teknisa\Libs\Util\DateUtil;
use NFePHP\Common\Keys;
use NFePHP\Common\DOMImproved as Dom;


class XmlNFeService {
    protected $transmissaoNfeFactory;
    protected $novoCodigo;
    protected $masks;
    protected $util;
    protected $corpoNF;
    protected $tpEmis;
    protected $xml;
    protected $timeZone = '-03:00';
    protected $infoCompra;
    /** @var ToolsNFe $nfeTools */
    protected $nfeTools;
    /** @var MakeNFe $nfe */
    protected $nfe;

    private $numeroNotaFiscal;
    private $rowCupomFiscal;
    private $tags = array();
    private $info;
    private $valorTotalBC           = 0.00;
    private $valorTotalICMS         = 0.00;
    private $valorTotalICMSDeson    = 0.00;
    private $valorTotalBCST         = 0.00;
    private $valorTotalICMSST       = 0.00;
    private $valorTotalIPI          = 0.00;
    private $valorTotalVSERV        = 0.00;
    private $valorTotalVBCISSQN     = 0.00;
    private $valorTotalVISS         = 0.00;
    private $valorTotalPISISSQN     = 0.00;
    private $valorTotalCOFINS       = 0.00;
    private $valorTotalCOFINSISSQN  = 0.00;
    private $valorTotalVII          = 0.00;
    private $valorTotalPIS          = 0.00;
    private $valorTotalVPROD        = 0.00;
    private $valorTotalPartDest     = 0.00;
    private $valorTotalPartRemet    = 0.00;
    private $valorAdicionalFCP      = 0.00;
    private $valorAdicionalFCPST    = 0.00;
    private $valorAdicionalFCPSTRet = 0.00;
    private $valorAdicionalFCPDest  = 0.00;
    

    const VERSAO = '4.00';
    const NOTAFISCAL_ENTRADA = 'E';

    public function __construct(TransmissaoNfeFactory $transmissaoNfeFactory, NewCode $novoCodigo, Utilities $util)
    {
        $this->transmissaoNfeFactory = $transmissaoNfeFactory;
        $this->novoCodigo = $novoCodigo;
        $this->masks = new Masks();
        $this->util = $util;
    }

    public function getNumeroNotaFiscal()
    {
        return $this->numeroNotaFiscal;
    }

    public function geraXML($nota, $corpoNF, $nfeTools, $tpEmis){
        $this->nfe = new Make();
        $this->corpoNF = $corpoNF;
        $this->nfeTools = $nfeTools;
        $this->nfeTools->config->cnpj = $this->corpoNF["NRINSJUREMIT"];
        $this->tpEmis = $tpEmis;
        $this->boGeraTagExporta = false;
        $this->numeroNotaFiscal = $this->geraNumeroNotaFiscal();
        $this->criaEstruturaXML();
        
        if($this->nfe->montaNFe()){
            $this->xml = $this->nfe->getXML();
            return array('xml' => $this->xml, 'chave' => $this->chave, 'numero' => $this->numeroNotaFiscal, 'errors' => $this->nfe->erros);
        };
        
        return false;
    }

    private function criaEstruturaXML(){
        $this->criaTagIdentificacao();
        $this->criaTagEmitente();
        $this->criaTagDestinatario();
        $this->criaTagEntrega();
        $this->criaTagAutXml();
        $this->criaTagItensNotafiscal();
    }
   
    private function criaTagIdentificacao()
    {
        $dtEmi = \DateTime::createFromFormat('d/m/Y H:i:s', $this->corpoNF["DTEMISSAO"], new \DateTimeZone('America/Sao_Paulo'));
        if($this->corpoNF["IDENTRSAIDOP"] !== Notafiscal::NOTA_FISCAL_ENTRADA){
            $paramgeral = Paramgeral::findOneByNrorg($this->corpoNF['NRORG']);

            if($paramgeral->getIdutdtproctnfe() === "S"){
                $objTipoOperac = \Geral\Entities\Tipooperac::getObjectByWhere("CDTIPOOPER = ? AND IDENTRSAIDOP = ?", array($this->corpoNF["CDTIPOOPER"], $this->corpoNF["IDENTRSAIDOP"]));
                $idtptransnfe = $objTipoOperac->getIdtptransnfe();
                if($idtptransnfe === "N" || $idtptransnfe === "C" || $idtptransnfe === null){
                    $dtSaiEnt = \DateTime::createFromFormat('d/m/Y H:i:s', $this->corpoNF["DTENTRSAID"], new \DateTimeZone('America/Sao_Paulo'));
                }else{
                    $dtEmi = \DateTime::createFromFormat('d/m/Y H:i:s', $this->nfeTools->config->dadosFilial->DTPROCESSA, new \DateTimeZone('America/Sao_Paulo'));

                    // $dtEntrSaid = new \Zend_Date($this->corpoNF["DTENTRSAID"], null, 'pt_BR');
                    $dtEntrSaid = DateUtil::getDataDeString($this->corpoNF["DTENTRSAID"], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                    // $dtProc = new \Zend_Date($this->nfeTools->config->dadosFilial->DTPROCESSA, null, 'pt_BR');
                    $dtProc = DateUtil::getDataDeString($this->nfeTools->config->dadosFilial->DTPROCESSA, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                    // Data fim nao pode ser menor que data inicio
                    // if($dtProc->compare($dtEntrSaid) < 0){
                    if(DateUtil::comparaDatas($dtProc, $dtEntrSaid)){
                        $dtSaiEnt = \DateTime::createFromFormat('d/m/Y H:i:s', $this->corpoNF["DTENTRSAID"], new \DateTimeZone('America/Sao_Paulo'));
                    }else{
                        $dtSaiEnt = \DateTime::createFromFormat('d/m/Y H:i:s', $this->nfeTools->config->dadosFilial->DTPROCESSA, new \DateTimeZone('America/Sao_Paulo'));
                    }
                }
            }else{
                $dtSaiEnt = \DateTime::createFromFormat('d/m/Y H:i:s', $this->corpoNF["DTENTRSAID"], new \DateTimeZone('America/Sao_Paulo'));
            }
        }
        //Dados da NFe - infNFe
        $cUF    = $this->nfeTools->getcUF($this->corpoNF['SGESTAEMIT']); //codigo numerico do estado
        $cNF    = substr($this->corpoNF["NRLANCTONF"], 2, 8); //numero aleatório da NF
        $natOp  = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMTIPOOPER"]))); //natureza da operação
        // $indPag = $this->getFormaPagamento($this->corpoNF["IDENTRSAIDOP"], $this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"], $this->corpoNF["IDORIGDESTNF"], \DateTime::createFromFormat('d/m/Y H:i:s', $this->corpoNF["DTENTRSAID"], new \DateTimeZone('America/Sao_Paulo'))); //0=Pagamento à vista; 1=Pagamento a prazo; 2=Outros
        $mod    = $this->corpoNF["CDMODDOCFISC"]; //modelo da NFe 55 ou 65 essa última NFCe
        $serie  = $this->corpoNF['CDSERIE']; //serie da NFe
        $nNF    = ltrim($this->numeroNotaFiscal, '0'); // numero da NFe
        
        
        //criada uma variável auxiliar para comparar se ele é igual ao número da nota fiscal, remove-se todos os zeros 
        $auxCNF = ltrim($cNF, '0');
        
        // caso o codigo numerico for igual a '00000000' é reatribuído o mesmo valor sem corta-lo;
        if ($cNF === '00000000') {
            $auxCNF = $cNF;
        }
        
        // Esse bloco verifica se a variável auxiliar(código numérico) sem os zeros a esquerda é igual ao número da nota fiscal, caso for o $cNF é somado +1. Para que não entre na rejeição 897;
        switch ($auxCNF){
            case ($nNF):
            case ('00000000'):
            case ('11111111'):
            case ('22222222'):
            case ('33333333'):
            case ('44444444'):
            case ('55555555'):
            case ('66666666'):
            case ('77777777'):
            case ('88888888'):
            case ('99999999'):
            case ('12345678'):
            case ('23456789'):
            case ('34567890'):
            case ('45678901'):
            case ('56789012'):
            case ('67890123'):
            case ('78901234'):
            case ('89012345'):
            case ('90123456'):
            case ('01234567'):
                $cNF = $cNF + 1;
                break;
        }
        
        //        $dhEmi = date("Y-m-d\TH:i:sP");//Formato: “AAAA-MM-DDThh:mm:ssTZD” (UTC - Universal Coordinated Time).
        $dhEmi = $dtEmi->format("Y-m-d") . 'T' . date('H:i:s') . $this->timeZone;
        
        //var_dump('new \DateTime()', new \DateTime());
        //die;
        //        $dhSaiEnt = date("Y-m-d\TH:i:sP");//Não informar este campo para a NFC-e.
        $dhSaiEnt = "";
        
        if (isset($dtSaiEnt)) {
            $dhSaiEnt = $dtSaiEnt->format("Y-m-d") . 'T' . date('H:i:s') . $this->timeZone;
        }
        
        $tpNF     = $this->corpoNF["IDENTRSAIDOP"] === self::NOTAFISCAL_ENTRADA ? "0" : "1";
        //        $idDest = '1'; //1=Operação interna; 2=Operação interestadual; 3=Operação com exterior.
        $cdfisopeitem = $this->transmissaoNfeFactory->getMaxCDFISOPEITEM($this->corpoNF['CDFILIAL'], $this->corpoNF['NRLANCTONF'], $this->corpoNF['NRORG']);
        if (in_array($cdfisopeitem[0], array('1', '5'))) {
            $idDest = '1';
        } elseif (in_array($cdfisopeitem[0], array('2', '6'))) {
            $idDest = '2';
        } else if(in_array($cdfisopeitem[0], array('3', '7'))){
            $idDest = '3';
            $this->boGeraTagExporta = true;
        }


        //Tem cliente da teknisa que o cadastro IBGE do municipio com 5 caracteres e outros com 7 caracteres
        if(strlen($this->masks->removeMascaraInscricao($this->corpoNF["CDMUNIIBGEEMIT"])) === 7){
            $cMunFG = trim($this->corpoNF["CDMUNIIBGEEMIT"]);
        }else{
            $cMunFG = trim($this->corpoNF["CDESTAIBGEEMIT"]).trim($this->corpoNF["CDMUNIIBGEEMIT"]);
        }
        $tpImp  = trim($this->corpoNF["IDTPIMPRESDANFE"]); //0=Sem geração de DANFE; 1=DANFE normal, Retrato; 2=DANFE normal, Paisagem;
        //3=DANFE Simplificado; 4=DANFE NFC-e; 5=DANFE NFC-e em mensagem eletrônica
        //(o envio de mensagem eletrônica pode ser feita de forma simultânea com a impressão do DANFE;
        //usar o tpImp=5 quando esta for a única forma de disponibilização do DANFE).

        $tpEmis = $this->tpEmis;
        //1=Emissão normal (não em contingência);
        //2=Contingência FS-IA, com impressão do DANFE em formulário de segurança;
        //3=Contingência SCAN (Sistema de Contingência do Ambiente Nacional);
        //4=Contingência DPEC (Declaração Prévia da Emissão em Contingência);
        //5=Contingência FS-DA, com impressão do DANFE em formulário de segurança;
        //6=Contingência SVC-AN (SEFAZ Virtual de Contingência do AN);
        //7=Contingência SVC-RS (SEFAZ Virtual de Contingência do RS);
        //9=Contingência off-line da NFC-e (as demais opções de contingência são válidas também para a NFC-e);
        //Nota: Para a NFC-e somente estão disponíveis e são válidas as opções de contingência 5 e 9.
        $tpAmb    = $this->nfeTools->config->tpAmb; //1=Produção; 2=Homologação
        // $finNFe   = '1'; //1=NF-e normal; 2=NF-e complementar; 3=NF-e de ajuste; 4=Devolução/Retorno.
        if($this->corpoNF["IDNOTADEVO"][0] === 'S') {
            $finNFe = "4";
        } else if($this->corpoNF["IDNFESTCANC"][0] === "S" || $this->corpoNF["IDNFAJUSTE"][0] === "S"){
            $finNFe = "3";
        }else{
            $finNFe = "1";
        }

        //0=Normal; 1=Consumidor final;
        if($this->corpoNF['IDORIGDESTNF'] == 'C' || $this->corpoNF['IDORIGDESTNF'] == 'F'){
            if($this->corpoNF['IDCONSFINAL'] == 'S')
                $indFinal = '1';
            else
                $indFinal = '0';
        }
        else
            $indFinal = '0';
        
        $indPres = $this->corpoNF['IDPRESENCIAL'];
        $intermed = null;
        
        if($indPres == 2 || $indPres == 3 || $indPres == 4 || $indPres == 9 ){
            $nt2020006 = $this->transmissaoNfeFactory->atualizaXMLNT2020006(); // Atualizar este bloco após 01/09/2021.
            
                 if($nt2020006['ATUALIZAPROD'] && $this->nfeTools->config->tpAmb == 1)    $intermed = $this->corpoNF['IDINTERMED'];
            else if($nt2020006['ATUALIZAHOMOLOG'] && $this->nfeTools->config->tpAmb == 2) $intermed = $this->corpoNF['IDINTERMED'];
            
        }
        
        //$indPres  = '9'; //0=Não se aplica (por exemplo, Nota Fiscal complementar ou de ajuste);
        //1=Operação presencial;
        //2=Operação não presencial, pela Internet;
        //3=Operação não presencial, Teleatendimento;
        //4=NFC-e em operação com entrega a domicílio;
        //9=Operação não presencial, outros.
        $procEmi = '0'; //0=Emissão de NF-e com aplicativo do contribuinte;
        //1=Emissão de NF-e avulsa pelo Fisco;
        //2=Emissão de NF-e avulsa, pelo contribuinte com seu certificado digital, através do site do Fisco;
        //3=Emissão NF-e pelo contribuinte com aplicativo fornecido pelo Fisco.
        $verProc = '4.0.50'; //versão do aplicativo emissor
        // Verificar parte de contingencia
        /*$dhCont  = isset($this->nfeTools->config->time)date('c', $this->nfeTools->tsContingencia); //entrada em contingência AAAA-MM-DDThh:mm:ssTZD
        $xJust   = $this->nfeTools->motivoContingencia; //Justificativa da entrada em contingência
*/
        //Numero e versão da NFe (infNFe)
        $ano  = date('y', strtotime($dhEmi));
        $mes  = date('m', strtotime($dhEmi));
        $cnpj = $this->nfeTools->config->cnpj;
        
        $this->chave = Keys::build($cUF, $ano, $mes, $cnpj, $mod, $serie, $nNF, $tpEmis, $cNF);

        $versao = self::VERSAO;
        
        $infNfe = new \stdClass();
        $infNfe->Id = "NFe".$this->chave;
        $infNfe->versao = $versao;
        
        $resp   = $this->nfe->taginfNFe($infNfe);
        
        // Tag Responsável Técnico
        $infRespTec = new \stdClass();
        $infRespTec->CNPJ       = '26269316000177';
        $infRespTec->xContato   = 'Wilson Lima de Paula';
        $infRespTec->email      = 'qualidade@teknisa.com';
        $infRespTec->fone       = '03121222300';
        
        $resp = $this->nfe->taginfRespTec($infRespTec);

        $cDV = substr($this->chave, -1); //Digito Verificador da Chave de Acesso da NF-e, o DV é calculado com a aplicação do algoritmo módulo 11 (base 2,9) da Chave de Acesso.

        //tag IDE
        $ide = new \stdClass();
        $ide->cUF = $cUF;
        $ide->cNF = $cNF;
        $ide->natOp = $natOp;
        
        // IndPag não existe na versão 4.00
        //$ide->indPag = $indPag;
        
        $ide->mod          = $mod;
        $ide->serie        = $serie;
        $ide->nNF          = $nNF;
        $ide->dhEmi        = $dhEmi;
        $ide->dhSaiEnt     = $dhSaiEnt;
        $ide->tpNF         = $tpNF;
        $ide->idDest       = $idDest;
        $ide->cMunFG       = $cMunFG;
        $ide->tpImp        = $tpImp;
        $ide->tpEmis       = $tpEmis;
        $ide->cDV          = $cDV;
        $ide->tpAmb        = $tpAmb;
        $ide->finNFe       = $finNFe;
        $ide->indFinal     = $indFinal;
        $ide->indPres      = $indPres;
        $ide->indIntermed  = $intermed;
        $ide->procEmi      = $procEmi;
        $ide->verProc      = $verProc;
        /*$ide->dhCont   = $dhCont;
        $ide->xJust    = $xJust;*/
        $resp = $this->nfe->tagide($ide);
        
        //refNFe NFe referenciada
        if($this->corpoNF['IDOPERDEVONF'] === 'S' || $this->corpoNF['CDTIPOOPER'] === '03' || $this->corpoNF['IDNFESTCANC'] === 'S'){
            $resultSetNotaDevolucao = $this->transmissaoNfeFactory->consultaDevolucaoNF($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
            foreach($resultSetNotaDevolucao as $rowNotaDevo){
                if(!empty($rowNotaDevo["NRACESSONFE"])){
                    //refNFe NFe referenciada
                    $refNFe = new \stdClass();
                    $refNFe->refNFe = $rowNotaDevo["NRACESSONFE"];
                    
                    $resp = $this->nfe->tagrefNFe($refNFe);
                }else{
                    //refNF Nota Fiscal 1A referenciada
                    $cUF = trim($this->corpoNF["CDESTAIBGEEMIT"]);
                    $AAMM = \DateTime::createFromFormat('d/m/Y H:i:s', $rowNotaDevo["DTEMISSAO"], new \DateTimeZone('America/Sao_Paulo'))->format("ym");
                    $CNPJ = trim($this->masks->removeMascaraInscricao($this->corpoNF["NRINSJUREMIT"]));
                    // Alteração 4.00
                    $mod = $rowNotaDevo["CDMODDOCFISC"];
                    $serie = $rowNotaDevo["CDSERIE"];
                    $nNF = empty($nNF) ? 1 : $nNF = ltrim($rowNotaDevo["NRNOTAFISC"], "0");
                    
                    $refNf = new \stdClass();
                    $refNf->cUF     = $cUF;
                    $refNf->AAMM    = $AAMM;
                    $refNf->CNPJ    = $CNPJ;
                    $refNf->mod     = $mod;
                    $refNf->serie   = $serie;
                    $refNf->nNF     = $nNF;

                    $resp = $this->nfe->tagrefNF($refNf);
                }
            }
        }

        //NFPref Nota Fiscal Produtor Rural referenciada
        //Verificando a existência de Notas Referenciadas para Produtor Rural
        $resultSetProdutorRural = $this->transmissaoNfeFactory->consultaNotaFiscalProdRural($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"], $this->corpoNF["NRORG"]);
        if ($resultSetProdutorRural){
            foreach($resultSetProdutorRural as $rowProdutorRural){
                if(!empty($rowProdutorRural["NRACESSONFE"])){
                    //refNFe NFe referenciada
                    $refNFe = new \stdClass();
                    $refNFe->refNFe =  $rowProdutorRural["NRACESSONFE"];
                    $resp = $this->nfe->tagrefNFe($refNFe);
                }
                else{
                    $CNPJ = '';
                    $CPF = '';
                    $cUF = $rowProdutorRural["CDESTADOIBGE"];
                    $AAMM = \DateTime::createFromFormat('d/m/Y H:i:s', $rowProdutorRural["DTEMISSAO"], new \DateTimeZone('America/Sao_Paulo'))->format("ym");
                    $IE = $rowProdutorRural["CDINSCESTA"];
                    $mod = $rowProdutorRural["CDMODDOCFISC"];
                    $serie = $rowProdutorRural["CDSERIE"];
                    $nNF = ltrim($rowProdutorRural["NRNOTAFISC"], "0");

                    if($rowProdutorRural['IDTPINSCJUR'] === "J"){
                        $CNPJ = trim($this->masks->removeMascaraInscricao($rowProdutorRural["NRINSJURIEST"]));
                    }elseif($rowProdutorRural['IDTPINSCJUR'] === "F"){
                        $CPF = trim($this->masks->removeMascaraInscricao($rowProdutorRural["NRINSJURIEST"]));
                    }
                    
                    $refNFP = new \stdClass();
                    $refNFP->cUF     = $cUF;
                    $refNFP->AAMM    = $AAMM;
                    $refNFP->CNPJ    = $CNPJ;
                    $refNFP->CPF     = $CPF;
                    $refNFP->IE      = $IE;
                    $refNFP->mod     = $mod;
                    $refNFP->serie   = $serie;
                    $refNFP->nNF     = $nNF;
                    
                    $resp = $this->nfe->tagrefNFP($refNFP);
                }
            }
        }

        //CTeref CTe referenciada
        //$refCTe = '12345678901234567890123456789012345678901234';
        //$resp = $nfe->tagrefCTe($refCTe);

        //ECFref ECF referenciada
        // Cupom Fiscal
        $consultaCupom = $this->transmissaoNfeFactory->consultaCupomFiscal($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"], $this->corpoNF["NRORG"]);
        if($consultaCupom && !empty($consultaCupom)) {
            foreach($consultaCupom as $rowCupomFiscal) {
                if(!empty($rowCupomFiscal['NRACESSONFCENF'])) {
                    //refNFe NFe referenciada
                    $refNFe = new \stdClass();
                    $refNFe->refNFe = $rowCupomFiscal["NRACESSONFCENF"];
                    
                    $resp = $this->nfe->tagrefNFe($refNFe);
                }
                else {
                    $refECF = new \stdClass();
                    $mod = "2D";
                    $nECF = $rowCupomFiscal["NRECFCUPOMNF"];
                    $nCOO = substr($rowCupomFiscal["NRCOOCUPOMNF"], -6);
                    
                    $refECF->mod  = $mod;
                    $refECF->nECF = $nECF;
                    $refECF->nCOO = $nCOO;
                    
                    $resp = $this->nfe->tagrefECF($refECF);
                }
            }
            $this->rowCupomFiscal = $consultaCupom[0];
        }
        $this->criaTagDocReferenciados();
    }
    
    private function criaTagDocReferenciados(){
        $resultSetChaveDeAcessoNFe = $this->transmissaoNfeFactory->consultaChaveDeAcessoNFe($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
        if($resultSetChaveDeAcessoNFe && !empty($resultSetChaveDeAcessoNFe)) {
            foreach($resultSetChaveDeAcessoNFe as $rowChaveDeAcessoNFe) {
                if(!empty($rowChaveDeAcessoNFe['CDREFNFE'])) {

                    $refNFe = new \stdClass();
                    $refNFe->refNFe = $rowChaveDeAcessoNFe["CDREFNFE"];
                    
                    $resp = $this->nfe->tagrefNFe($refNFe);
                }
            }
        }
        
        $resultSetNotaFiscalMod1A = $this->transmissaoNfeFactory->consultaNotaFiscalMod1A($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
        if($resultSetNotaFiscalMod1A && !empty($resultSetNotaFiscalMod1A)) {
            foreach($resultSetNotaFiscalMod1A as $rowNotaFiscalMod1A) {
                if(!empty($rowNotaFiscalMod1A['CDCUF'])) {
                    $refNF = new \stdClass();
                    $cUF = $rowNotaFiscalMod1A["CDCUF"];

                    $AAMM = \DateTime::createFromFormat('d/m/Y H:i:s', $rowNotaFiscalMod1A["DTEMISSAO"], new \DateTimeZone('America/Sao_Paulo'))->format("ym");
                
                    $CNPJ = trim($this->masks->removeMascaraInscricao($rowNotaFiscalMod1A["NRINSJUREMITDOC"]));

                    $mod = $rowNotaFiscalMod1A['CDMOD'];

                    $serie = $rowNotaFiscalMod1A['CDSERIE'];
                    
                    $nNF = $rowNotaFiscalMod1A['NRNF'];
                    
                    $refNF->cUF  = $cUF;
                    $refNF->AAMM = $AAMM;
                    $refNF->CNPJ = $CNPJ;
                    $refNF->mod  = $mod;
                    $refNF->serie = $serie;
                    $refNF->nNF = $nNF;
                    
                    $resp = $this->nfe->tagrefNF($refNF);
                    
                }    
                
            }
        }
        $resultSetconsultaNotaFiscalProdutorRural = $this->transmissaoNfeFactory->consultaNotaFiscalProdutorRural($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
        if($resultSetconsultaNotaFiscalProdutorRural && !empty($resultSetconsultaNotaFiscalProdutorRural)) {
            foreach($resultSetconsultaNotaFiscalProdutorRural as $rowNotaFiscalProdutorRural) {
                if(!empty($rowNotaFiscalProdutorRural['CDCUF'])) {
                    $refNFP = new \stdClass();
                    $cUF = $rowNotaFiscalProdutorRural["CDCUF"];

                    $AAMM = \DateTime::createFromFormat('d/m/Y H:i:s', $rowNotaFiscalProdutorRural["DTEMISSAO"], new \DateTimeZone('America/Sao_Paulo'))->format("ym");
                
                    if(strlen($rowNotaFiscalProdutorRural['NRINSJUREMITDOC']) == 14){
                        $CNPJ = $rowNotaFiscalProdutorRural["NRINSJUREMITDOC"];
                        $refNFP->CNPJ = $CNPJ;
                    }elseif(strlen($rowNotaFiscalProdutorRural['NRINSJUREMITDOC']) == 11){
                        $CPF = $rowNotaFiscalProdutorRural["NRINSJUREMITDOC"];
                        $refNFP->CPF = $CPF;
                    }
                    
                    $IE = $rowNotaFiscalProdutorRural['NRIE'];

                    $mod = $rowNotaFiscalProdutorRural['CDMOD'];

                    $serie = $rowNotaFiscalProdutorRural['CDSERIE'];
                    
                    $nNF = $rowNotaFiscalProdutorRural['NRNF'];
                    
                    $refNFP->cUF  = $cUF;
                    $refNFP->AAMM = $AAMM;
                    $refNFP->IE = $IE;
                    $refNFP->mod  = $mod;
                    $refNFP->serie = $serie;
                    $refNFP->nNF = $nNF;
                    
                    $resp = $this->nfe->tagrefNFP($refNFP);    
                }
            }
        }
        $resultSetconsultaInfoCTe = $this->transmissaoNfeFactory->consultaInfoCTe($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
        if($resultSetconsultaInfoCTe && !empty($resultSetconsultaInfoCTe)) {
            foreach($resultSetconsultaInfoCTe as $rowInfoCTe) {
                if(!empty($rowInfoCTe['CDREFCTE'])) {
                    $refCTe = new \stdClass();
                    $refCTe->refCTe = $rowInfoCTe["CDREFCTE"];
                    
                    $resp = $this->nfe->tagrefCTe($refCTe);
                }
            }
        }
                    
        $resultSetconsultaInfoCupomFiscal = $this->transmissaoNfeFactory->consultaInfoCupomFiscal($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
        if($resultSetconsultaInfoCupomFiscal && !empty($resultSetconsultaInfoCupomFiscal)) {
            foreach($resultSetconsultaInfoCupomFiscal as $rowInfoCupomFiscal) {
                if(!empty($rowInfoCupomFiscal['CDMOD'])) {
                    $refECF = new \stdClass();
                    
                    $mod = $rowInfoCupomFiscal["CDMOD"];
                    
                    $nECF = $rowInfoCupomFiscal["NRNECF"];
                    
                    $nCOO = $rowInfoCupomFiscal["NRNCOO"];
                    
                    $refECF->mod  = $mod;
                    $refECF->nECF = $nECF;
                    $refECF->nCOO = $nCOO;
                    
                    $resp = $this->nfe->tagrefECF($refECF);
                }
            }
        }

        
        
        
    }
    private function criaTagEmitente(){
        //Dados do emitente - (Importando dados do config.json)
        $xNome  = $this->nfeTools->config->razaosocial;
        $xFant  = $this->nfeTools->config->nomefantasia;
        $IE     = $this->nfeTools->config->ie;
        $IEST   = $this->nfeTools->config->iest;
        $IM     = $this->nfeTools->config->im;
        $CNAE   = $this->nfeTools->config->cnae;
        $CRT    = $this->nfeTools->config->regime;

        $CNPJ = '';
        $CPF = ''; // Utilizado para CPF na nota
        if ($this->corpoNF["IDTPIJUREMIT"] === 'F'){
            $CPF = str_pad($this->corpoNF["NRINSJUREMIT"], 11, "0");
        }else{
            if ($this->corpoNF["CDPAISBACENEMIT"] === '1058') {
                $CNPJ = $this->masks->removeMascaraInscricao($this->corpoNF["NRINSJUREMIT"]);
            } else {
                //Caso o emitente nao seja do brasil (caso de notas de importação)
                $CNPJ = $this->masks->removeMascaraInscricao($this->corpoNF["NRINSJUREMIT"]);
                $CNPJ = str_pad(trim(substr($CNPJ, strlen($CNPJ)-14, 14)), 14, "0");
            }
        }

        if(!empty($this->corpoNF["CDINSCESTAST"])){
            $IEST = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->corpoNF["CDINSCESTAST"])));
        }
        
        if(!empty($this->corpoNF["NRINSCESTEMIT"])){
            $IE = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->corpoNF["NRINSCESTEMIT"])));
        }

        $resultSetItemProImpos = $this->transmissaoNfeFactory->consultaItemProdutoImpostoNFE($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"], "31", $this->corpoNF["NRORG"]);
        if ($resultSetItemProImpos && !empty($resultSetItemProImpos)) {
            // Nao eh necessario verificar se os mesmos estao vazios no array, pois tal validacao eh feita na class ValidaNFE, ln - 310/322
            $IM = trim($this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->corpoNF["CDINSCMUNI"])));
            $CNAE = str_pad(trim($this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->corpoNF["CDCNAE"]))), 7, "0");
        }
                
        $tagEmit = new \stdClass();
        $tagEmit->xNome = $xNome;
        $tagEmit->xFant = $xFant;
        $tagEmit->IE    = ($IE == '' || $IE == 'ISENTA') ? 'ISENTO' : $IE ;
        $tagEmit->IEST  = $IEST;
        $tagEmit->IM    = $IM;
        $tagEmit->CNAE  = $CNAE;
        $tagEmit->CRT   = $CRT;
        $tagEmit->CNPJ  = $CNPJ;
        $tagEmit->CPF   = $CPF;

        $resp = $this->nfe->tagemit($tagEmit);
        /*if($tagEmit->IE == 'ISENTO') {
            var_dump($this->nfe->emit->getElementsByTagName("CRT")->item(0));die;
        }*/
        
        $this->criaTagEnderecoEmitente();
    }

    private function criaTagEnderecoEmitente(){
        //endereço do emitente
        $xLgr = '';
        if (!empty($this->corpoNF["DSENDEEMIT"])) {
            $xLgr = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["DSENDEEMIT"])));
        }
        $nro = 's/n';
        $xCpl = '';
        $xBairro = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMBAIREMIT"])));
        if ($this->corpoNF["CDPAISBACENEMIT"] === "1058") {
            if (strlen($this->masks->removeMascaraInscricao($this->corpoNF["CDMUNIIBGEEMIT"])) === 7) {
                $cMun = trim($this->corpoNF["CDMUNIIBGEEMIT"]);
            } else {
                $cMun = trim($this->corpoNF["CDESTAIBGEEMIT"]) . trim($this->corpoNF["CDMUNIIBGEEMIT"]);
            }

            $xMun = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMMUNIEMIT"])));
            $UF = trim($this->corpoNF["SGESTAEMIT"]);
        } else {
            //Operacao do exterior
            $cMun = '9999999';
            $xMun = 'EXTERIOR';
            $UF = 'EX';
        }
        $CEP = str_pad($this->masks->removeCharsFromString($this->corpoNF["NRCEPEMIT"]), 8, "0");
        $cPais = trim($this->corpoNF["CDPAISBACENEMIT"]);
        $xPais = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMPAISEMIT"])));
        $fone = $this->masks->removeCharsFromString($this->corpoNF["NRTELEEMIT"]);
        
        $tagEnderEmit = new \stdClass();
        $tagEnderEmit->xLgr = $xLgr;
        $tagEnderEmit->nro = $nro;
        $tagEnderEmit->xCpl = $xCpl;
        $tagEnderEmit->xBairro = $xBairro;
        $tagEnderEmit->cMun = $cMun;
        $tagEnderEmit->xMun = $xMun;
        $tagEnderEmit->UF = $UF;
        $tagEnderEmit->CEP = $CEP;
        $tagEnderEmit->cPais = $cPais;
        $tagEnderEmit->xPais = $xPais;
        
        if($fone && strlen($fone) > 8) {
            if ($this->corpoNF["CDPAISBACENEMIT"] === "1058") {
                if(strlen($fone) >= 11) {
                    $fone = substr($fone, strlen($fone) - 11, 11);
                }
            }
            else {
                $fone = substr($fone, 0, 14);
            }
            $tagEnderEmit->fone = $fone;
        }
        
        $resp = $this->nfe->tagenderEmit($tagEnderEmit);
    }

    private function criaTagDestinatario(){
        //destinatário
        $CPF = '';
        $CNPJ = '';
        $IE = '';
        $IM = '';
        $ISUF = "";
        $email = '';
        $idEstrangeiro = '';

        if($this->rowCupomFiscal){
            if($this->nfeTools->config->tpAmb == "1"){
                if($this->corpoNF["IDBRINDETPOPER"] === "N"){
                    $xNome = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->rowCupomFiscal["NMRAZSODESNF"]))), 0, 60);
                }else{
                    if($this->corpoNF["IDENTRSAIDOP"] === "S"){
                        $xNome = 'Diversos – Remessa de Brindes';
                    }else{
                        $xNome = 'Diversos – Brindes';
                    }
                }
            }else{
                $xNome = 'NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
            }
            
            $CNPJ = '';
            $CPF = ''; // Utilizado para CPF na nota
            
            //Verificar se o destinatario e do exterior
            //Codigo 1058 é o cóigo do brasil de acordo com o codigo do banco central
            //Quando for cliente do Exterior nao passar o CPF e nem o CNPJ
            
            if($this->rowCupomFiscal["CDPAISBACEN"] === "1058"){
                if($this->rowCupomFiscal["IDTPIJUDESNF"] === "F" || (strlen($this->rowCupomFiscal["NRINSJUDESNF"]) == 11) ){
                    $CPF = trim($this->masks->removeMascaraInscricao($this->rowCupomFiscal["NRINSJUDESNF"]));
                }else{
                    $CNPJ = trim($this->masks->removeMascaraInscricao($this->rowCupomFiscal["NRINSJUDESNF"]));
                }
            }else{
                $idEstrangeiro = trim($this->masks->removeMascaraInscricao($this->rowCupomFiscal["NRINSJUDESNF"]));
            }

            if($this->corpoNF['IDCONSFINAL']  === 'S') {
                $indIEDest = '9';
                if($this->rowCupomFiscal['NRINSESTDESNF']) {
                    $IE = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->rowCupomFiscal['NRINSESTDESNF'])));
                }
            }else if($this->rowCupomFiscal['NRINSESTDESNF']) {
                if(strpos("ISENTO",strtoupper($this->rowCupomFiscal['NRINSESTDESNF'])) === 0 || strpos("ISENTA", strtoupper($this->rowCupomFiscal['NRINSESTDESNF'])) === 0) {
                    $indIEDest = '2';
                    
                }else {
                    $indIEDest = '1';
                    $IE = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->rowCupomFiscal['NRINSESTDESNF'])));
                }
            }
            else {
                $indIEDest = '9';
            }
            
            $tagDest = new \stdClass();
            $tagDest->xNome = $xNome;
            $tagDest->indIEDest = $indIEDest;
            $tagDest->IE = $IE;
            $tagDest->ISUF = $ISUF;
            $tagDest->IM = $IM;
            $tagDest->email = $email;
            $tagDest->CNPJ = $CNPJ;
            $tagDest->CPF = $CPF;
            $tagDest->idEstrangeiro = $idEstrangeiro;
            
            $resp = $this->nfe->tagdest($tagDest);

            $this->criaTagEnderecoDestinatarioCupomFiscal($this->rowCupomFiscal);
        }
        else{
            $ISUF = trim($this->corpoNF["NRINSCSUFRAMA"]);
            if(!empty($this->corpoNF["DSEMAILDEST"])){
                $dsEmail = trim($this->corpoNF['DSEMAILDEST']);
                $email = str_replace(",",";", $dsEmail);
                $posVirg = strpos($email, ";");
                if($posVirg !== false) {
                    $email = substr($email, 0, $posVirg);
                }
                // length(stEmail) <= 60
                $email = strlen($email) > 60 ? "" : $email;
            }

            if($this->nfeTools->config->tpAmb == "1"){
                if($this->corpoNF["IDBRINDETPOPER"] === "N"){
                    $xNome = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMRAZSOCCLIE"])));
                }else{
                    if($this->corpoNF["IDENTRSAIDOP"] === "S"){
                        $xNome = 'Diversos – Remessa de Brindes';
                    }else{
                        $xNome = 'Diversos – Brindes';
                    }
                }
            }
            else{
                $xNome = 'NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
            }

            $CNPJ = '';
            $CPF = ''; // Utilizado para CPF na nota
            
            if($this->corpoNF["CDPAISBACENCLIE"] === "1058"){
                if($this->corpoNF["IDTPIJURCLIE"] === "F"){
                    $CPF = $this->masks->removeMascaraInscricao($this->corpoNF["NRINSJURCLIE"]);
                }else{
                    $CNPJ = $this->masks->removeMascaraInscricao($this->corpoNF["NRINSJURCLIE"]);
                }
            }else{
                $idEstrangeiro = $this->masks->removeMascaraInscricao($this->corpoNF['NRINSJURCLIE']);
            }
            
            if($this->corpoNF['IDCONSFINAL']  === 'S') {
                $indIEDest = '9';
                if($this->corpoNF['NRINSCESTCLIE']) {
                    $IE = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->corpoNF["NRINSCESTCLIE"])));
                }
            }
            else if($this->corpoNF['NRINSCESTCLIE']){
                if(strpos("ISENTO",strtoupper($this->corpoNF["NRINSCESTCLIE"])) === 0 || strpos("ISENTA", strtoupper($this->corpoNF["NRINSCESTCLIE"])) === 0) {
                    $indIEDest = '2';
                }
                else{
                    $indIEDest = '1';
                    $IE = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->corpoNF["NRINSCESTCLIE"])));
                }
            }
            else{
                $indIEDest = '9';
            }

            $tagDest = new \stdClass();
            $tagDest->xNome = $xNome;
            $tagDest->indIEDest = $indIEDest;
            $tagDest->IE = $IE;
            $tagDest->ISUF = $ISUF;
            $tagDest->IM = $IM;
            $tagDest->email = $email;
            $tagDest->CNPJ = $CNPJ;
            $tagDest->CPF = $CPF;
            $tagDest->idEstrangeiro = $idEstrangeiro;

            $resp = $this->nfe->tagdest($tagDest);

            $this->criaTagEnderecoDestinatario();
        }
    }
    
    private function criaTagAutXml() {
        $inscricoesAutorizadas = $this->transmissaoNfeFactory->getInscricoesAutorizadas($this->corpoNF["CDEMPRESA"], $this->corpoNF["NRINSCESTEMIT"]);

        foreach ($inscricoesAutorizadas as $aut) {
            if (strlen($aut["NRINSJURAUTOXML"]) == 14) {
                $tagAutXml = new \stdClass();
                $tagAutXml->CNPJ = $aut["NRINSJURAUTOXML"];
                $this->nfe->tagautXML($tagAutXml);
            } else {
                $tagAutXml = new \stdClass();
                $tagAutXml->CPF = $aut["NRINSJURAUTOXML"];
                $this->nfe->tagautXML($tagAutXml);
            }
        }
    }

    private function criaTagEnderecoDestinatario(){
        //Endereço do destinatário
        $xLgr = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["DSENDECLIE"])));
        $nro = 's/n';
        $xCpl = '';
        if (!empty($this->corpoNF["DSCOMPENDCLI"])) {
            $xCpl = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["DSCOMPENDCLI"])));
        }
        $xBairro = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMBAIRCLIE"])));
        $xMun = "";
        if ($this->corpoNF["CDPAISBACENCLIE"] === "1058") {
            if (strlen($this->masks->removeMascaraInscricao($this->corpoNF["CDMUNIIBGECLIE"])) === 7) {
                $cMun = $this->corpoNF["CDMUNIIBGECLIE"];
            } else {
                $cMun = $this->corpoNF["CDESTAIBGECLIE"] . $this->corpoNF["CDMUNIIBGECLIE"];
            }
            if (!empty($this->corpoNF["NMMUNICLIE"])) {
                $xMun = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMMUNICLIE"])));
            }
            $UF = trim($this->corpoNF["SGESTACLIE"]);
        } else {
            $cMun = '9999999';
            $xMun = 'EXTERIOR';
            $UF = 'EX';
        }
        $CEP = '';
        if (!empty($this->corpoNF["NRCEPCLIE"])) {
            $CEP = str_replace(" ", "", str_pad($this->masks->removeMascaraInscricao($this->corpoNF["NRCEPCLIE"]), 8, "0"));
        }
        $cPais = trim($this->corpoNF["CDPAISBACENCLIE"]);
        if (!empty($this->corpoNF["NMPAISCLIE"])) {
            $xPais = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMPAISCLIE"])));
        }
        $tagEnderDest = new \stdClass();
        
        
        
        $fone = $this->masks->removeCharsFromString($this->corpoNF["NRTELECLIE"]);
        
        $tagEnderDest->xLgr = $xLgr;
        $tagEnderDest->nro = $nro;
        $tagEnderDest->xCpl = $xCpl;
        $tagEnderDest->xBairro = $xBairro;
        $tagEnderDest->cMun = $cMun;
        $tagEnderDest->xMun = $xMun;
        $tagEnderDest->UF = $UF;
        $tagEnderDest->CEP = $CEP;
        $tagEnderDest->cPais = $cPais;
        $tagEnderDest->xPais = $xPais;
        
        if($fone && strlen($fone) > 8) {
            if ($this->corpoNF["CDPAISBACENCLIE"] === "1058") {
                if(strlen($fone) >= 11) {
                    $fone = substr($fone, strlen($fone) - 11, 11);
                }
            }
            else {
                $fone = substr($fone, 0, 14);
            }
            $tagEnderDest->fone = $fone;
        }
        
        $resp = $this->nfe->tagenderDest($tagEnderDest);
    }
    
    private function criaTagEntrega() {
        $dadosEnt = [
            "CNPJ" => '', "CPF" => '', "xNome" => '', "xLgr"  => '', "nro"   => '.', "xCpl" => '', "xBairro" => '', "cMun" => '',
            "xMun" => '', "UF"  => '', "CEP"   => '', "cPais" => '', "xPais" => '' , "fone" => '', "email"   => '', "IE"   => ''];
            
        $idPresencial = isset($this->corpoNF['IDPRESENCIAL']) ? $this->corpoNF['IDPRESENCIAL'] : null;
            
        if($idPresencial === '1') {
            $dadosEnt = $this->criaTagEntregaEnderecoFilial($dadosEnt);
        }
        else {
            $nfPedido = $this->transmissaoNfeFactory->getNFPedido($this->corpoNF['CDFILIAL'], $this->corpoNF['NRLANCTONF']);
            
            if(isset($nfPedido['CDFILIALPED'], $nfPedido['NRPEDIDO'])) {
                $dadosEnt = $this->criaTagEntregaEnderecoPedido($dadosEnt, $nfPedido);
            }
            else {
                $dadosEnt = $this->criaTagEntregaEnderecoEntrega($dadosEnt);
            }
        }
        
        if($dadosEnt) {
            $tagentrega = new \stdClass();
            $tagentrega->CNPJ       = $dadosEnt['CNPJ'];
            $tagentrega->CPF        = $dadosEnt['CPF'];
            $tagentrega->xNome      = $dadosEnt['xNome'];
            $tagentrega->xLgr       = $dadosEnt['xLgr'];
            $tagentrega->nro        = $dadosEnt['nro'];
            $tagentrega->xCpl       = $dadosEnt['xCpl'];
            $tagentrega->xBairro    = $dadosEnt['xBairro'];
            $tagentrega->cMun       = $dadosEnt['cMun'];
            $tagentrega->xMun       = $dadosEnt['xMun'];
            $tagentrega->UF         = $dadosEnt['UF'];
            $tagentrega->CEP        = $dadosEnt['CEP'];
            $tagentrega->cPais      = $dadosEnt['cPais'];
            $tagentrega->xPais      = $dadosEnt['xPais'];
            $tagentrega->fone       = $dadosEnt['fone'];
            $tagentrega->email      = $dadosEnt['email'];
            $tagentrega->IE         = $dadosEnt['IE'];

            $resp = $this->nfe->tagentrega($tagentrega);
        }
    }
    
    private function criaTagEntregaEnderecoFilial($dadosEnt) {
        
        $endFilial = $this->transmissaoNfeFactory->getEndFilial($this->corpoNF['CDFILIEMIT']);
        
        /*Sapore - Lá o endereço para emissão da filial padrão é o da inscrição estadual, o que fazia com
          que esse endereço fosse gerado como o de entrega. A validação abaixo foi criada para que o endereço da
          filial emitente seja gerado como de entrega quando o endereço para emissão for o da inscrição estadual.*/
        
        if($endFilial) {
            if($endFilial['IDENDEMISFIL'] === 'I') {
                if($this->corpoNF['CDPAISBACENEMIT'] === '1058') {
                    if($this->corpoNF['IDTPIJUREMIT'] === 'F') {
                        $dadosEnt['CPF'] = trim($this->masks->removeMascaraInscricao($this->corpoNF['NRINSJUREMIT']));
                    } else {
                        $dadosEnt['CNPJ'] = trim($this->masks->removeMascaraInscricao($this->corpoNF['NRINSJUREMIT']));
                    }
                }
                
                if(!empty($this->corpoNF['NMRAZSOCEMIT'])) {
                    $dadosEnt['xNome'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['NMRAZSOCEMIT']))), 0, 60);
                }
                if(!empty($endFilial['DSENDEFILI'])) {
                    $dadosEnt['xLgr'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endFilial['DSENDEFILI']))), 0, 60);
                }
                if(!empty($endFilial['DSCOMPENDFIL'])) {
                    $dadosEnt['xCpl'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endFilial['DSCOMPENDFIL']))), 0, 60);
                }
                $dadosEnt['xBairro'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endFilial['NMBAIRFILI']))), 0, 60);
                
                if($this->corpoNF['CDPAISBACENEMIT'] === '1058') {
                    if(strlen($this->masks->removeMascaraInscricao($endFilial['CDMUNICIBGE'])) === 7) {
                        $dadosEnt['cMun'] = trim($endFilial['CDMUNICIBGE']);
                    } else {
                        $dadosEnt['cMun'] = trim($endFilial['CDESTADOIBGE']) . trim($endFilial['CDMUNICIBGE']);
                    }
                    
                    if(!empty($endFilial['NMMUNICIPIO'])) {
                        $dadosEnt['xMun'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endFilial['NMMUNICIPIO']))), 0, 60);
                    }
                    $dadosEnt['UF'] = substr(trim($endFilial['SGESTADO']), 0, 2);
                    
                    if(!empty($endFilial['NRCEPFILI'])) {
                        $dadosEnt['CEP'] = str_replace(" ", "", str_pad($this->masks->removeMascaraInscricao($endFilial['NRCEPFILI']), 8, "0"));
                    }
                    if(!empty($endFilial['CDPAISBACEN'])) {
                        $dadosEnt['cPais'] = trim($endFilial['CDPAISBACEN']);
                    }
                    if(!empty($endFilial['NMPAIS'])) {
                        $dadosEnt['xPais'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endFilial['NMPAIS']))), 0, 60);
                    }
                }
                else { //Operação do exterior
                    $dadosEnt['cMun'] = '9999999';
                    $dadosEnt['xMun'] = 'EXTERIOR';
                    $dadosEnt['UF'] = 'EX';
                }
                
                if(!empty($endFilial['NRTELEFILI'])) {
                    $dadosEnt['fone'] = substr($this->masks->removeCharsFromString($endFilial['NRTELEFILI']), 0, 14);
                }
                if(!empty($endFilial['DSEMAILFILI'])) {
                    $dsEmail = trim($endFilial['DSEMAILFILI']);
                    $email = str_replace(",",";", $dsEmail);
                    $posVirg = strpos($email, ";");
                    
                    if($posVirg !== false) {
                        $email = substr($email, 0, $posVirg);
                    }
                    $dadosEnt['email'] = strlen($email) > 60 ? "" : $email;
                }
                if(!empty($this->corpoNF['SGESTACLIE']) && !empty($endFilial['SGESTADO'])) {
                    if($this->corpoNF['SGESTACLIE'] === $endFilial['SGESTADO']) {
                        if($this->corpoNF['NRINSCESTCLIE']) {
                            $dadosEnt['IE'] = trim($this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->corpoNF["NRINSCESTCLIE"])));
                        }
                    }
                }
                return $dadosEnt;
            }
            else {
                if($this->corpoNF['CDPAISBACENEMIT'] === '1058') {
                    if($this->corpoNF['IDTPIJUREMIT'] === 'F') {
                        $dadosEnt['CPF'] = trim($this->masks->removeMascaraInscricao($this->corpoNF['NRINSJUREMIT']));
                    } else {
                        $dadosEnt['CNPJ'] = trim($this->masks->removeMascaraInscricao($this->corpoNF['NRINSJUREMIT']));
                    }
                }
                
                if(!empty($this->corpoNF['NMRAZSOCEMIT'])) {
                    $dadosEnt['xNome'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['NMRAZSOCEMIT']))), 0, 60);
                }
                if(!empty($this->corpoNF['DSENDEEMIT'])) {
                    $dadosEnt['xLgr'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['DSENDEEMIT']))), 0, 60);
                }
                if(!empty($this->corpoNF['DSCOMPENDEMIT'])) {
                    $dadosEnt['xCpl'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['DSCOMPENDEMIT']))), 0, 60);
                }
                $dadosEnt['xBairro'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['NMBAIREMIT']))), 0, 60);
                
                if($this->corpoNF['CDPAISBACENEMIT'] === '1058') {
                    if(strlen($this->masks->removeMascaraInscricao($this->corpoNF['CDMUNIIBGEEMIT'])) === 7) {
                        $dadosEnt['cMun'] = trim($this->corpoNF['CDMUNIIBGEEMIT']);
                    } else {
                        $dadosEnt['cMun'] = trim($this->corpoNF['CDESTAIBGEEMIT']) . trim($this->corpoNF['CDMUNIIBGEEMIT']);
                    }
                    
                    if(!empty($this->corpoNF['NMMUNIEMIT'])) {
                        $dadosEnt['xMun'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['NMMUNIEMIT']))), 0, 60);
                    }
                    $dadosEnt['UF'] = substr(trim($this->corpoNF['SGESTAEMIT']), 0, 2);
                    
                    if(!empty($this->corpoNF['NRCEPEMIT'])) {
                        $dadosEnt['CEP'] = str_replace(" ", "", str_pad($this->masks->removeMascaraInscricao($this->corpoNF['NRCEPEMIT']), 8, "0"));
                    }
                    if(!empty($this->corpoNF['CDPAISBACENEMIT'])) {
                        $dadosEnt['cPais'] = trim($this->corpoNF['CDPAISBACENEMIT']);
                    }
                    if(!empty($this->corpoNF['NMPAISEMIT'])) {
                        $dadosEnt['xPais'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['NMPAISEMIT']))), 0, 60);
                    }
                }
                else { //Operação do exterior
                    $dadosEnt['cMun'] = '9999999';
                    $dadosEnt['xMun'] = 'EXTERIOR';
                    $dadosEnt['UF'] = 'EX';
                }
                
                if(!empty($this->corpoNF['NRTELEEMIT'])) {
                    $dadosEnt['fone'] = substr($this->masks->removeCharsFromString($this->corpoNF['NRTELEEMIT']), 0, 14);
                }
                //Identificação do Emitente não possui informação de e-mail.
                //IE não é enviada porque aqui as informações de entrega são do próprio emitente sendo desnecessária enviá-la.
                
                return $dadosEnt;
            } //fim else
            
        } //fim if($endFilial)
        
        return false;
    }
    
    private function criaTagEntregaEnderecoPedido($dadosEnt, $nfPedido) {
        
        $endPedido = $this->transmissaoNfeFactory->getEndPedido($nfPedido['CDFILIALPED'], $nfPedido['NRPEDIDO']);
        
        if($endPedido) {
            if($endPedido['CDPAISBACEN'] === '1058') {
                if($endPedido['IDTPIJURCLIE'] === 'F') {
                    $dadosEnt['CPF'] = trim($this->masks->removeMascaraInscricao($endPedido['NRINSJURCLIE']));
                } else {
                    $dadosEnt['CNPJ'] = trim($this->masks->removeMascaraInscricao($endPedido['NRINSJURCLIE']));
                }
            }
            
            if(!empty($this->corpoNF['NMRAZSOCCLIE'])) {
                $dadosEnt['xNome'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['NMRAZSOCCLIE']))), 0, 60);
            }
            if(!empty($endPedido['DSENDEPED'])) {
                $dadosEnt['xLgr'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endPedido['DSENDEPED']))), 0, 60);
            }
            //Endereço do Pedido não possui informação de complemento.
            
            $dadosEnt['xBairro'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endPedido['NMBAIRROPED']))), 0, 60);
            
            if($endPedido['CDPAISBACEN'] === '1058') {
                if(strlen($this->masks->removeMascaraInscricao($endPedido['CDMUNICIBGE'])) === 7) {
                    $dadosEnt['cMun'] = trim($endPedido['CDMUNICIBGE']);
                } else {
                    $dadosEnt['cMun'] = trim($endPedido['CDESTADOIBGE']) . trim($endPedido['CDMUNICIBGE']);
                }
                
                if(!empty($endPedido['NMMUNIPED'])) {
                    $dadosEnt['xMun'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endPedido['NMMUNIPED']))), 0, 60);
                }
                $dadosEnt['UF'] = substr(trim($endPedido['SGESTADOPED']), 0, 2);
                
                if(!empty($endPedido['NRCEPPED'])) {
                    $dadosEnt['CEP'] = str_replace(" ", "", str_pad($this->masks->removeMascaraInscricao($endPedido['NRCEPPED']), 8, "0"));
                }
                if(!empty($endPedido['CDPAISBACEN'])) {
                    $dadosEnt['cPais'] = trim($endPedido['CDPAISBACEN']);
                }
                if(!empty($endPedido['NMPAISPED'])) {
                    $dadosEnt['xPais'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endPedido['NMPAISPED']))), 0, 60);
                }
            }
            else { //Operação do exterior
                $dadosEnt['cMun'] = '9999999';
                $dadosEnt['xMun'] = 'EXTERIOR';
                $dadosEnt['UF'] = 'EX';
            }
            
            if(!empty($endPedido['NRTELEPED'])) {
                $dadosEnt['fone'] = substr($this->masks->removeCharsFromString($endPedido['NRTELEPED']), 0, 14);
            }
            //Endereço do Pedido não possui informação de e-mail.
            
            if(!empty($this->corpoNF['SGESTACLIE']) && !empty($endPedido['SGESTADOPED'])) {
                if($this->corpoNF['SGESTACLIE'] === $endPedido['SGESTADOPED']) {
                    if($this->corpoNF['NRINSCESTCLIE']) {
                        $dadosEnt['IE'] = trim($this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->corpoNF["NRINSCESTCLIE"])));
                    }
                }
            }
            return $dadosEnt;
        }
        return false;
    }
    
    private function criaTagEntregaEnderecoEntrega($dadosEnt) {
        
        $endEntrega = $this->transmissaoNfeFactory->getEndEntrega($this->corpoNF['CDFILIAL'], $this->corpoNF['NRLANCTONF']);
        
        if($endEntrega) {
            if($this->corpoNF['CDPAISBACENCLIE'] === '1058') {
                if($this->corpoNF['IDTPIJURCLIE'] === 'F') {
                    $dadosEnt['CPF'] = trim($this->masks->removeMascaraInscricao($this->corpoNF['NRINSJURCLIE']));
                } else {
                    $dadosEnt['CNPJ'] = trim($this->masks->removeMascaraInscricao($this->corpoNF['NRINSJURCLIE']));
                }
            }
            
            if(!empty($this->corpoNF['NMRAZSOCCLIE'])) {
                $dadosEnt['xNome'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['NMRAZSOCCLIE']))), 0, 60);
            }
            $dadosEnt['xLgr'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endEntrega['LOGRADOURO']))), 0, 60);
            
            if(!empty($endEntrega['COMPLEMENTO'])) {
                $dadosEnt['xCpl'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endEntrega['COMPLEMENTO']))), 0, 60);
            }
            $dadosEnt['xBairro'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endEntrega['BAIRRO']))), 0, 60);
            
            if($this->corpoNF['CDPAISBACENCLIE'] === '1058') {
                if(strlen($this->masks->removeMascaraInscricao($endEntrega['CDIBGEMUNI'])) === 7) {
                    $dadosEnt['cMun'] = trim($endEntrega['CDIBGEMUNI']);
                } else {
                    $dadosEnt['cMun'] = trim($endEntrega['CDESTADOIBGE']) . trim($endEntrega['CDIBGEMUNI']);
                }
                
                if(!empty($endEntrega['MUNICIPIO'])) {
                    $dadosEnt['xMun'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endEntrega['MUNICIPIO']))), 0, 60);
                }
                $dadosEnt['UF'] = substr(trim($endEntrega['UF']), 0, 2);
                
                if(!empty($endEntrega['NRCEP'])) {
                    $dadosEnt['CEP'] = str_replace(" ", "", str_pad($this->masks->removeMascaraInscricao($endEntrega['NRCEP']), 8, "0"));
                }
                if(!empty($endEntrega['CDPAISBACEN'])) {
                    $dadosEnt['cPais'] = trim($endEntrega['CDPAISBACEN']);
                }
                if(!empty($endEntrega['NMPAIS'])) {
                    $dadosEnt['xPais'] = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($endEntrega['NMPAIS']))), 0, 60);
                }
            }
            else { //Operação do exterior
                $dadosEnt['cMun'] = '9999999';
                $dadosEnt['xMun'] = 'EXTERIOR';
                $dadosEnt['UF'] = 'EX';
            }
            
            if(!empty($endEntrega['NRTELEFONE'])) {
                $dadosEnt['fone'] = substr($this->masks->removeCharsFromString($endEntrega['NRTELEFONE']), 0, 14);
            }
            if(!empty($endEntrega['EMAIL'])) {
                $dsEmail = trim($endEntrega['EMAIL']);
                $email = str_replace(",",";", $dsEmail);
                $posVirg = strpos($email, ";");
                
                if($posVirg !== false) {
                    $email = substr($email, 0, $posVirg);
                }
                $dadosEnt['email'] = strlen($email) > 60 ? "" : $email;
            }
            if(!empty($this->corpoNF['SGESTACLIE']) && !empty($endEntrega['UF'])) {
                if($this->corpoNF['SGESTACLIE'] === $endEntrega['UF']) {
                    if($this->corpoNF['NRINSCESTCLIE']) {
                        $dadosEnt['IE'] = trim($this->masks->removeMascaraInscricao($this->masks->removeAcentos($this->corpoNF["NRINSCESTCLIE"])));
                    }
                }
            }
            return $dadosEnt;
        }
        return false;
    }
    
    //Identificação do local de retirada (se diferente do emitente)
    //$CNPJ = '12345678901234';
    //$CPF = '';
    //$xLgr = 'Rua Vanish';
    //$nro = '000';
    //$xCpl = 'Ghost';
    //$xBairro = 'Assombrado';
    //$cMun = '3509502';
    //$xMun = 'Campinas';
    //$UF = 'SP';
    //$resp = $nfe->tagretirada($CNPJ, $CPF, $xLgr, $nro, $xCpl, $xBairro, $cMun, $xMun, $UF);
    
    //Identificação dos autorizados para fazer o download da NFe (somente versão 3.1)
    /*$aAut = array('23401454000170');
    foreach ($aAut as $aut) {
        if (strlen($aut) == 14) {
            $resp = $nfe->tagautXML($aut);
        } else {
            $resp = $nfe->tagautXML('', $aut);
        }
    }*/

    private function criaTagEnderecoDestinatarioCupomFiscal($resultSetCupomFiscal)
    {
        //Preenche o endereço do destinatario
        $CEP = "";
        $fone = "";
        $xCpl = "";
        $xLgr = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($resultSetCupomFiscal["DSENDEDESTNF"])));
        $nro = ".";
        $xBairro = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($resultSetCupomFiscal["NMBAIRDESTNF"])));
        if ($resultSetCupomFiscal["CDPAISBACEN"] === "1058") {
            if (strlen($this->masks->removeMascaraInscricao($resultSetCupomFiscal["CDMUNICIBGE"])) === 7) {
                $cMun = $resultSetCupomFiscal["CDMUNICIBGE"];
            } else {
                $cMun = $resultSetCupomFiscal["CDESTADOIBGE"] . $resultSetCupomFiscal["CDMUNICIBGE"];
            }

            $xMun = "";
            if (!empty($resultSetCupomFiscal["NMMUNICIPIO"])) {
                $xMun = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($resultSetCupomFiscal["NMMUNICIPIO"])));
            }

            $UF = trim($resultSetCupomFiscal["SGESTADO"]);
            
            if (!empty($resultSetCupomFiscal["NRCEPDESTNF"])) {
                $CEP = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($resultSetCupomFiscal["NRCEPDESTNF"])));
            }
            
        } else {
            $cMun = '9999999';
            $xMun = 'EXTERIOR';
            $UF = 'EX';
        }

        $cPais = trim($resultSetCupomFiscal["CDPAISBACEN"]);
        if (!empty($resultSetCupomFiscal["NMPAIS"])) {
            $xPais = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($resultSetCupomFiscal["NMPAIS"])));
        }
        
        $tagEnderDest = new \stdClass();
        
        $tagEnderDest->xLgr = $xLgr;
        $tagEnderDest->nro = $nro;
        $tagEnderDest->xCpl = $xCpl;
        $tagEnderDest->xBairro = $xBairro;
        $tagEnderDest->cMun = $cMun;
        $tagEnderDest->xMun = $xMun;
        $tagEnderDest->UF = $UF;
        $tagEnderDest->CEP = $CEP;
        $tagEnderDest->cPais = $cPais;
        $tagEnderDest->xPais = $xPais;
        $tagEnderDest->fone = $fone;
   
        $resp = $this->nfe->tagenderDest($tagEnderDest);
    }

    private function criaTagItensNotafiscal(){
        $cdfisopeitem = $this->transmissaoNfeFactory->getMaxCDFISOPEITEM($this->corpoNF['CDFILIAL'], $this->corpoNF['NRLANCTONF'], $this->corpoNF['NRORG']);
        
        if (in_array($cdfisopeitem[0], array('1', '5'))) {
                $idDest = '1';
            } elseif (in_array($cdfisopeitem[0], array('2', '6'))) {
                $idDest = '2';
            } else {
                $idDest = '3';
            }
            
        //0=Normal; 1=Consumidor final;
        if($this->corpoNF['IDORIGDESTNF'] == 'C' || $this->corpoNF['IDORIGDESTNF'] == 'F'){
            if($this->corpoNF['IDCONSFINAL'] == 'S')
                $indFinal = '1';
            else
                $indFinal = '0';
        }
        else {
            $indFinal = '0';
        }
        
        if($idDest == '2' && $indFinal == '1') {
            $fcp = true;
        }
        else {
            $fcp = false;
        }
        
        // Grupo do detalhamento de Produtos e Serviços da NF-e [infNFe] [H]
        //$infoProd = "";
        //$dadosadic = Dadosadic::findByCdfilialAndNrlanctonfAndNrorg($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"], $this->corpoNF["NRORG"]);
        
        //if($dadosadic instanceof Dadosadic){
            /** @var Dadosadic $dados */
        /*    foreach($dadosadic as $dados){
                if( strpos("NUMERO DA ORDEM DO PEDIDO", strtoupper($dados->getDsdadoadic())) ||
                    strpos("NÚMERO DA ORDEM DO PEDIDO", strtoupper($dados->getDsdadoadic())) ){
                    $infoProd = substr($dados->getDsdadoadic(), strpos("|", $dados->getDsdadoadic()), strlen(trim($dados->getDsdadoadic())));
                }

                $this->info = $this->util->convertISOToUTF8(trim($this->masks->removeAcentos($this->info . " " . $dados->getDsdadoadic())));
            }
        }*/
        
        $itemprod = $this->transmissaoNfeFactory->consultaItemNFE($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"], $this->corpoNF["CDPAISCLIE"], $this->corpoNF["SGESTACLIE"]);
        $i = 1;
        foreach($itemprod as $rowItemProd){
            $nItem = $i;
            $impostos = $this->transmissaoNfeFactory->consultaDadosImpostosNFE($rowItemProd["CDFILIAL"],$rowItemProd["NRLANCTONF"],$rowItemProd["NRSEQITPROD"]);
            
            $this->criaTagProdutoServico($nItem, $rowItemProd, $impostos);
            
            if(!empty($impostos)) {
                $this->criaTagCEST($nItem, $rowItemProd, $impostos);
            }

            $prodFili = $this->corpoNF["IDENTRSAIDOP"] === 'E' ? $this->transmissaoNfeFactory->getProdFiliTrans($this->corpoNF["CDFILIRECE"], $rowItemProd["CDPRODUTO"])
                                                               : $this->transmissaoNfeFactory->getProdFiliTrans($this->corpoNF["CDFILIEMIT"], $rowItemProd["CDPRODUTO"]);

            if($prodFili || $rowItemProd["IDSEGMENTOPROD"] === "MEDI") {
                $tipoOperTrans = $this->transmissaoNfeFactory->getTipoOperTrans($this->corpoNF["CDTIPOOPER"], $this->corpoNF["IDENTRSAIDOP"]);
                
                if(($prodFili["IDCNTRESTOQ"] == 'S' && $prodFili["IDUTCTRLLOTE"] == 'S' && $prodFili["IDPRODCOMRASTRO"] == 'S' && $tipoOperTrans["IDATUESTTPOP"] == 'S')
                    || $rowItemProd["IDSEGMENTOPROD"] === "MEDI") {
                    $this->criaTagRastro($nItem, $rowItemProd);
                }
            }
            
            $combs = $this->transmissaoNfeFactory->getNotaCombs($rowItemProd["CDFILIAL"], $rowItemProd["NRLANCTONF"], $rowItemProd["NRSEQITPROD"]);
            
            if(!empty($combs)) {
                $this->criaTagCombs($nItem, $combs);
                
                if(isset($combs['NRBICO']) && isset($combs['NRTANQUE']) && isset($combs['VRENCERRAINI']) && isset($combs['VRENCERRAFIM']))
                    $this->criaTagEncerrante($nItem, $combs);
            }
            
            /*if ($rowItemProd["IDSEGMENTOPROD"] === "MEDI" && $this->corpoNF["CDTIPOOPER"] != '03') {
                $this->criaTagDetalheMedicamentoEMateriasPrimasFarmaceuticas($nItem, $rowItemProd);
                $this->criaTagRastro($nItem, $rowItemProd);
            }*/
            
            if (!empty($impostos)) {
                //@todo remover foreach $vTotTrib quando atualiar o fechamento da nota (VRTOTATRIB)
                 $vTotTrib = 0;
                foreach($impostos as $impo){
                    if($impo["CDFISCALIT"] == '1')  $vTotTrib += ($impo['VRIMPOSTOIT'] + $impo['VRIMPOSTODIFERIT']);
                }
                
                $tagImposto = new \stdClass();
                
                $tagImposto->item = $nItem;
                $tagImposto->vTotTrib = number_format($this->util->decimalToFloat($vTotTrib), 2, '.', '');
                
                $this->nfe->tagimposto($tagImposto);
//                $this->nfe->tagimposto($nItem, number_format($this->util->decimalToFloat($rowItemProd['VRTOTATRIB]']), 2, '.', ''));

                //O grupo ISSQN é mutuamente exclusivo com os grupos ICMS, IPI e II,
                //isto é se ISSQN for informado os grupos ICMS, IPI e II não serão informados e vice-versa (v2.0).
                //Imposto.ISSQN
                $aux = array_search("31", array_column($impostos, "CDIMPOSTONFE"));
                $ISSQN = $aux !== false ? $impostos[$aux] : '';
    
                if($ISSQN){
                    $rowISSQN = $ISSQN->first();
                    $this->criaTagISSQN($nItem, $rowItemProd, $rowISSQN);
                } else {
                    $this->valorTotalVPROD += $this->util->decimalToFloat($rowItemProd["VRTOTAITEMFLOAT"]);
                    //Imposto.ICMS
                    $aux = array_search("00", array_column($impostos, "CDIMPOSTONFE"));
                    $ICMS = $aux !== false ? $impostos[$aux] : '';
                    
                    if ($ICMS) {
                        $this->criaTagICMS($nItem, $rowItemProd, $ICMS, $impostos, !$fcp);

                        // Operações Interestaduais
                        $this->criaTagICMSUfDest($nItem, $ICMS, $fcp);
                        
                    }
                    //Imposto.IPI
                    $aux = array_search("10", array_column($impostos, "CDIMPOSTONFE"));
                    $IPI = $aux !== false ? $impostos[$aux] : '';
                    //Para o campo 251 do manual esta sendo passado o valor fixo de '999'
    
                    if ($IPI) {
                        $this->criaTagIPI($nItem, $IPI);
                    }

                    //Imposto.II
                    $aux = array_search("20", array_column($impostos, "CDIMPOSTONFE"));
                    $II = $aux !== false ? $impostos[$aux] : '';
                    if ($II) {
                        $this->criaTagII($nItem, $II, $impostos);
                    }
                }

                //Imposto.PIS
                $aux = array_search("50", array_column($impostos, "CDIMPOSTONFE"));
                $PIS = $aux !== false ? $impostos[$aux] : '';
                if ($PIS) {
                    $this->criaTagPIS($nItem, $PIS, $ISSQN, $impostos);
                }

                //Imposto.COFINS
                $aux = array_search("60", array_column($impostos, "CDIMPOSTONFE"));
                $COFINS = $aux !== false ? $impostos[$aux] : '';
                if ($COFINS) {
                    $this->criaTagCOFINS($nItem, $COFINS, $rowItemProd, $impostos);
                }
                $i++;
            }
            
            //Monta as informações referentes aos dados adicionais 
            $nrpedprod = $this->masks->removeCharsFromString($this->masks->removeAcentos($rowItemProd["NRPEDPROD"]));
            $vrTotaImpDeson = "";
            $infoAdProd = "";
            $infAdProd = null;
            
            if(sizeof($this->transmissaoNfeFactory->consultaDesoneraItem($rowItemProd["CDFILIAL"],$rowItemProd["NRLANCTONF"],$rowItemProd["NRSEQITPROD"])) !== 0) {
                 $vrTotaImpDeson = $this->transmissaoNfeFactory->consultaDesoneraItem($rowItemProd["CDFILIAL"],$rowItemProd["NRLANCTONF"],$rowItemProd["NRSEQITPROD"])[0]["VRTOTAIMPDESON"];
            }
            
            if ($vrTotaImpDeson > 0) {
            	if (!empty($infoAdProd) ) {
            		$infoAdProd = $infoAdProd.' | ';
            	}
            	$infoAdProd = $infoAdProd.' Imposto Desonerado: R$ '.$this->util->decimalToFloat($vrTotaImpDeson, 2, ".", "");
            }
            
            if(!empty($rowItemProd["DSINFADICPROD"])) {
            	if(!empty($infoAdProd)) {
                    $infoAdProd = $infoAdProd.' | ' ;
            	}
            	$infoAdProd = $infoAdProd.trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($rowItemProd["DSINFADICPROD"])));
            }
            
            if(!empty($infoAdProd)) {
                $infoAdProd = trim(substr($infoAdProd, 0,  500));
                $infAdProd = new \stdClass();
                $infAdProd->item = $nItem;
                $infAdProd->infAdProd = $infoAdProd;
                $resp = $this->nfe->taginfAdProd($infAdProd);
            }
        }
        
        //Tag criaTagVeiculoTransportadora foi retirada do método criaTagTransportadora para gerar os dados referentes ao peso mesmo que não tenha sido informado Transportadora
        $this->criaTagDadosTotais();
        $this->criaTagTransportadora();
        $this->criaTagVeiculoTransportadora();
        $this->criaTagCobranca();
        $this->criaTagInformacoesIntermediador();
        $this->criaTagInformacoesAdicionais();
        $this->criaTagInfoCompras();
    }

    private function criaTagProdutoServico($nItem, $rowItemProd, $impostos){
        
        $cProd = trim($rowItemProd["CDPRODEMIS"]);
        
        if($rowItemProd["CDBARPRODUTO"]) {
            $codigoBarras = strlen($rowItemProd["CDBARPRODUTO"]) <= 7 ? "SEM GTIN" : trim($rowItemProd["CDBARPRODUTO"]);
        }
        else {
            $codigoBarras = "SEM GTIN";
        }
        $cEAN = $codigoBarras;
        
        $xProd = trim(str_replace(chr(13), " ", str_replace(chr(10), " ", $this->masks->removeAcentos($rowItemProd["DSPRODEMIS"]))));
        $EXTIPI = "";
        $NVE = '';
        $CEST = '';
        $NCM = "";
        $gravaIndEscala = false;
        
        if (!empty($rowItemProd["NCM"]) && strlen(trim($this->masks->removeMascaraInscricao($rowItemProd["NCM"]))) >= 2) {
            $NCM = trim($this->masks->removeMascaraInscricao($rowItemProd["NCM"]));
            /** @var Itemproduto $objItemproduto */
            $objItemproduto = Itemproduto::findOneByCdfilialAndNrlanctonfAndNrseqitprod($rowItemProd["CDFILIAL"], $rowItemProd["NRLANCTONF"], $rowItemProd["NRSEQITPROD"]);
            $objItemproduto->setCdprodncmtrans($NCM);
        }
        
        $CFOP = $rowItemProd["CFOP"];
        $uCom = trim($rowItemProd["SGUNIDADE"]);
        $qCom = trim($rowItemProd["QTITEMPROD"]);
        $vUnCom = $rowItemProd["VRUNITITEM"];
        $vProd = $rowItemProd["VRTOTAITEM"];
        $cEANTrib = $codigoBarras;
        $uTrib = $rowItemProd["SGUNIDADETRIB"];
        $qTrib = $rowItemProd["QTITEMPROD"];
        $vUnTrib = $rowItemProd["VRUNITITEM"];
        $vFrete = "";
        $vSeg = "";
        $vDesc = "";
        $vOutro = "";
        $cBenef = "";
        
        $prodCBenef = $this->transmissaoNfeFactory->getCBenefItem($rowItemProd["CDFILIAL"], $rowItemProd["NRLANCTONF"], $rowItemProd["NRSEQITPROD"]);
        if($prodCBenef) {
            if($prodCBenef["CDBENEFFISCAL"]) {
                $cBenef = $prodCBenef["CDBENEFFISCAL"];
            }
        }
           
        if(sizeof($this->transmissaoNfeFactory->consultaDesoneraItem($rowItemProd["CDFILIAL"],$rowItemProd["NRLANCTONF"],$rowItemProd["NRSEQITPROD"])) !== 0) {
            $rowDesonera = $this->transmissaoNfeFactory->consultaDesoneraItem($rowItemProd["CDFILIAL"], $rowItemProd["NRLANCTONF"], $rowItemProd["NRSEQITPROD"])[0];
        } else {
            $rowDesonera = 0;
        } 
        
        if ($this->util->decimalToFloat($rowItemProd["VRFRETEITEM"]) > 0)
            $vFrete = number_format($this->util->decimalToFloat($rowItemProd["VRFRETE"]), 2, ".", "");
        if ($this->util->decimalToFloat($rowItemProd["VRSEGURITEM"]) > 0)
            $vSeg = number_format($this->util->decimalToFloat($rowItemProd["VRSEGURO"]), 2, ".", "");
        if ($this->util->decimalToFloat($rowItemProd["VRENCARGODESC"]) > 0 || $this->util->decimalToFloat($rowDesonera["VRIMPOSTOIT"]) > 0)
            $vDesc = number_format(($this->util->decimalToFloat($rowItemProd["VRENCARGODESC"]) + $this->util->decimalToFloat($rowDesonera["VRIMPOSTOIT"])), 2, ".", "");
        if ($this->util->decimalToFloat($rowItemProd["VROUTRASDESPITEM"]) > 0)
            $vOutro = $rowItemProd["OUTRASDESPESAS"];

        $xPed = "";
        $nrpedprod = $this->masks->removeCharsFromString($this->masks->removeAcentos($rowItemProd["NRPEDPROD"]));
        if (!empty($nrpedprod)) {
            $xPed = $nrpedprod;
        }

        $nrseqpedprod = $this->masks->removeCharsFromString($this->masks->removeAcentos($rowItemProd["NRSEQPEDPROD"]));
        $nItemPed = "";
        if (!empty($nrseqpedprod)) {
            $nItemPed = $nrseqpedprod;
        }
        //0 = Valor do item não compõe o valor total da NF
        //1 = Valor do item compõe o valor total da NF
        //No sistema teknisa, todos os produtos compõe o valor tota da nota fiscal
        $indTot = "1";
        $nFCI = '';

        $tagProd = new \stdClass();

        $tagProd->item = $nItem;
        $tagProd->cProd = $cProd;
        $tagProd->cEAN = $cEAN;
        $tagProd->xProd = $xProd;
        $tagProd->NCM = $NCM;
        $tagProd->EXTIPI = $EXTIPI;
        $tagProd->CFOP = $CFOP;
        $tagProd->uCom = $uCom;
        $tagProd->qCom = $qCom;
        $tagProd->vUnCom = $vUnCom;
        $tagProd->vProd = $vProd;
        $tagProd->cEANTrib = $cEANTrib;
        $tagProd->uTrib = $uTrib;
        $tagProd->qTrib = $qTrib;
        $tagProd->vUnTrib = $vUnTrib;
        $tagProd->vFrete = $vFrete;
        $tagProd->vSeg = $vSeg;
        if($vDesc) $tagProd->vDesc = $vDesc;
        if($cBenef) $tagProd->cBenef = $cBenef;
        $tagProd->vOutro = $vOutro;
        $tagProd->indTot = $indTot;
        $tagProd->xPed = $xPed;
        $tagProd->nItemPed = $nItemPed;
        $tagProd->nFCI = $nFCI;

        $resp = $this->nfe->tagprod($tagProd);

        if (!empty($rowItemProd["CFOP"])) {
            if (strlen($rowItemProd["CFOP"]) === 4) {
                if ($rowItemProd["CFOP"][0] === "3") {
                    $params = array(
                        "P_CDFILIAL" => $rowItemProd["CDFILIAL"],
                        "P_NRLANCTONF" => $rowItemProd["NRLANCTONF"],
                        "P_NRSEQITPROD" => $rowItemProd["NRSEQITPROD"]
                    );
                    $objDecimpnfe = $this->transmissaoNfeFactory->getDecimpnfeInfo($params);
                    if (!empty($objDecimpnfe[0])) {
                        $this->criaTagDeclaraImporta($nItem, $objDecimpnfe[0]);
                        $this->criaTagAdicoes($nItem, $params, $objDecimpnfe[0]['NRDOCIMPNFE']);
                    }
                }
            }
        }
    }
    
    private function criaTagCEST($nItem, $rowItemProd, $impostos) {
        
        $aux = array_search('00', array_column($impostos, "CDIMPOSTONFE"));
        $rowImpostrib = $aux !== false ? $impostos[$aux] : '';   
        $CEST = '';
        // Preenche CEST
        if($rowImpostrib) {
            if($rowImpostrib["CDIMPOSTRIB"] != null) {
                $aux = array_search($rowImpostrib["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
                $imposto = $aux !== false ? $impostos[$aux] : '';  
                if($imposto) {
                    if($imposto["CDCEST"] != null) {
                        $CEST = str_replace(array(".", "/"), '', $imposto["CDCEST"]);
                    }
                    else if($rowItemProd["CDCEST"] != null){
                        $CEST = str_replace(array(".", "/"), '', $rowItemProd["CDCEST"]);
                    }
                }
                else {
                    if($rowImpostrib["CDCEST"] != null) {
                        $CEST = str_replace(array(".", "/"), '', $rowImpostrib["CDCEST"]);
                    }
                    else if($rowItemProd["CDCEST"] != null){
                        $CEST = str_replace(array(".", "/"), '', $rowItemProd["CDCEST"]);
                    }
                }
            }
            else {
                $impTrib = $this->transmissaoNfeFactory->impostoTrib($rowItemProd["CDFILIAL"], $rowItemProd["NRLANCTONF"]);
                
                if(isset($impTrib[0])) {
                    $aux = array_search($impTrib[0]["CDIMPOSTO"], array_column($impostos, "CDIMPOSTO"));
                    $imposto = $aux !== false ? $impostos[$aux] : '';
                    
                    if($imposto) {
                        if($imposto["CDCEST"] != null) {
                            $CEST = $imposto["CDCEST"];
                        }
                        else if($rowItemProd["CDCEST"] != null){
                            $CEST = str_replace(array(".", "/"), '', $rowItemProd["CDCEST"]);
                        }
                    }
                    else if($rowItemProd["CDCEST"] != null){
                        $CEST = str_replace(array(".", "/"), '', $rowItemProd["CDCEST"]);
                    }
                }
                else {
                    if($rowImpostrib["CDCEST"] != null) {
                            $CEST = $rowImpostrib["CDCEST"];
                        }
                    else if($rowItemProd["CDCEST"] != null){
                        $CEST = str_replace(array(".", "/"), '', $rowItemProd["CDCEST"]);
                    }
                }
            }
        }

       if($CEST != null) {
            $tagCEST = new \stdClass();
            $tagCEST->item = $nItem;
            $tagCEST->CEST = $CEST;
            $tagCEST->indEscala = 'S';
            $resp = $this->nfe->tagCEST($tagCEST);
        } 
    
    }

    //DI - Declaração de Importação
    private function criaTagDeclaraImporta($nItem, $objDecimpnfe)
    {
        $nDI = $objDecimpnfe['NRDOCIMPNFE'];
        $dDI = \DateTime::createFromFormat('d/m/Y H:i:s', $objDecimpnfe['DTREGIMPNFE'], new \DateTimeZone('America/Sao_Paulo'))->format("Y-m-d");
        $xLocDesemb = $objDecimpnfe['DSLOCEMBIMP'];
        $UFDesemb = $objDecimpnfe['SGUFLOCEMBIMP'];
        $dDesemb = \DateTime::createFromFormat('d/m/Y H:i:s', $objDecimpnfe['DTDESEMBIMP'], new \DateTimeZone('America/Sao_Paulo'))->format("Y-m-d");
        $tpViaTransp = ltrim($objDecimpnfe['IDVIATRANSPORTE'], '0');
        $tpIntermedio = $objDecimpnfe['IDFORMAIMPORTA'];
        $cExportador = $objDecimpnfe['CDEXPIMPNFE'];
        $CNPJ = '';
        if(!is_null($objDecimpnfe['NRINSJURADQUIRE'])){
            $CNPJ = $objDecimpnfe['NRINSJURADQUIRE'];
        }
        $UFTerceiro = '';
        if(!is_null($objDecimpnfe['SGESTADOADQUIRE'])){
            $UFTerceiro = $objDecimpnfe['SGESTADOADQUIRE'];
        }
        $vAFRMM = '0.00';
        if(!is_null($objDecimpnfe['VRAFRMM'])){
            $vAFRMM = $objDecimpnfe['VRAFRMM'];
        }
        
        
        $tagDI = new \stdClass();
        
        $tagDI->item = $nItem;
        $tagDI->nDI = $nDI;
        $tagDI->dDI = $dDI;
        $tagDI->xLocDesemb = $xLocDesemb;
        $tagDI->UFDesemb = $UFDesemb;
        $tagDI->dDesemb = $dDesemb;
        $tagDI->tpViaTransp = $tpViaTransp;
        $tagDI->vAFRMM = $vAFRMM;
        $tagDI->tpIntermedio = $tpIntermedio;
        $tagDI->CNPJ = $CNPJ;
        $tagDI->UFTerceiro = $UFTerceiro;
        $tagDI->cExportador = $cExportador;

        $resp  = $this->nfe->tagDI($tagDI);
    }

    //adi - Adições
    private function criaTagAdicoes($nItem, $params, $nDI)
    {
        $arObjAdicionais = $this->transmissaoNfeFactory->getAdicdecimpnfeInfo($params);
        if (!empty($arObjAdicionais)) {
            $cFabricante = '';
            $nDraw = '0';
            $vDescDI = '';
            foreach ($arObjAdicionais as $objAdicional) {
                $nAdicao = $objAdicional['NRADICDECIMP'];
                $nSeqAdic = $objAdicional['NRSEQADICDEC'];
                if ($objAdicional['CDFABRICADIC'] !== "") {
                    $cFabricante = $objAdicional['CDFABRICADIC'];
                }
                if ($objAdicional['VRDESCADICDEC'] != 0) {
                     $vDescDI = $objAdicional['VRDESCADICDEC'];
                }
                if ($objAdicional['NRDRAWBACK'] != 0) {
                     $nDraw = $objAdicional['NRDRAWBACK'];
                }
                
                $tagadi = new \stdClass();
                
                $tagadi->item = $nItem;
                $tagadi->nDI = $nDI;
                $tagadi->nAdicao = $nAdicao;
                $tagadi->nSeqAdic = $nSeqAdic;
                $tagadi->cFabricante = $cFabricante;
                $tagadi->vDescDI = $vDescDI;
                $tagadi->nDraw = $nDraw;
                
                $resp = $this->nfe->tagadi($tagadi);
            }
        }
    }

    //detExport
    //$nItem = '2';
    //$nDraw = '9393939';
    //$exportInd = '1';
    //$nRE = '2222';
    //$chNFe = '1234567890123456789012345678901234';
    //$qExport = '100';
    //$resp = $nfe->tagdetExport($nItem, $nDraw, $exportInd, $nRE, $chNFe, $qExport);

    /**
     * Grupo do detalhamento de Medicamentos e de matériasprimas farmacêuticas [prod]
     * @param $rowItemProd
     */
    private function criaTagDetalheMedicamentoEMateriasPrimasFarmaceuticas($nItem, $rowItemProd)
    {
        $nLote = "";
        $dFab  = "";
        $dVal  = "";
        $qLote = "";

        $cProdANVISA = $rowItemProd["NRREGISTROANVISA"];
            
        if ($this->corpoNF["IDORIGDESTNF"] === "C") {
            $paramCliente = $this->corpoNF["CDCLIENTE"];
        } elseif ($this->corpoNF["IDORIGDESTNF"] === "F") {
            $paramCliente = $this->corpoNF["CDFORNECED"];
        } elseif ($this->corpoNF["IDORIGDESTNF"] === "S") {
            $paramCliente = $this->corpoNF["CDFILIRECE"];
        } else {
            $paramCliente = $this->corpoNF["CDFILIEMIT"];
        }

        $row = $this->transmissaoNfeFactory->consultaInscricaoSuframaNFE($paramCliente);
        if (!empty($row["NRSUFRAMA"])) {
            $rowMaxSuframa = $this->transmissaoNfeFactory->consultaMaxValorSuframa($this->corpoNF["CDPAISCLIE"], "SF");
            if (!empty($rowMaxSuframa["VRMAXVENDA"])) {
                $vPMC = number_format($this->util->decimalToFloat($rowMaxSuframa["VRMAXVENDA"]), 2, ".", "");
            } else {
                $vPMC = "0.00";
            }
        } else {
            if (!empty($rowItemProd["VRMAXVENDA"])) {
                $vPMC = number_format($this->util->decimalToFloat($rowItemProd["VRMAXVENDA"]), 2, ".", "");
            } else {
                $vPMC = "0.00";
            }
        }

        $tagMed = new \stdClass();
        $tagRastro->item = $nItem;
        $tagMed->vPMC = $vPMC;
        $tagMed->cProdANVISA = $cProdANVISA;
        $tagMeg->xMotivoIsencao = $rowItemProd["DSISENCAOANVISA"];

        $resp = $this->nfe->tagmed($tagMed);
    }
    
    private function criaTagRastro($nItem, $rowItemProd) {
        $nLote = "";
        $dFab  = "";
        $dVal  = "";
        $qLote = "";

        $nLote = trim($rowItemProd["NRLOTEESTQ"]);
        $qLote = $rowItemProd["QTPRODLOTE"];
        $dFab  = $rowItemProd["DTFABRPROD"];
        $dVal = $rowItemProd["DTVALILOTE"];
        
        $tagRastro = new \stdClass();
        $tagRastro->item = $nItem;
        $tagRastro->nLote = $nLote;
        $tagRastro->qLote = $qLote;
        $tagRastro->dFab = $dFab;
        $tagRastro->dVal = $dVal;

        $tagRastro->cAgreg = $rowItemProd["CDAGREGACAO"];

        $resp = $this->nfe->tagRastro($tagRastro);
        

    }
    
    private function criaTagCombs($nItem, $combs) {
        $cProdANP = $combs['CDPRODANP'];
        $descANP = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($combs['DSPRODANP']))), 0, 60);
        $UFCons = $combs['SGESTADO'];
        $pGLP = $combs['VRPERCENTGLP'];
        $pGNn = $combs['VRPERCENTGASNAC'];
        $pGNi = $combs['VRPERCENTGASIMP'];
        $vPart = $combs['VRPARTIDA'];
        $codif = $combs['CDCODIF'];
        $qTemp = $combs['QTCOMBFATUAMB'];
        $qBCProd = $combs['VALORBASECALC'];
        $vAliqProd = $combs['VALORALIQUOTA'];
        $vCIDE = $combs['VALORCIDE'];
        
        $tagCombs = new \stdClass();
        $tagCombs->item      = $nItem;
        $tagCombs->cProdANP  = $cProdANP;
        $tagCombs->descANP   = $descANP;
        $tagCombs->UFCons    = $UFCons;
        $tagCombs->pGLP      = $pGLP;
        $tagCombs->pGNn      = $pGNn;
        $tagCombs->pGNi      = $pGNi;
        $tagCombs->vPart     = $vPart;
        $tagCombs->CODIF     = $codif;
        $tagCombs->qTemp     = $qTemp;
        $tagCombs->qBCProd   = $qBCProd;
        $tagCombs->vAliqProd = $vAliqProd;
        $tagCombs->vCIDE     = $vCIDE;
        
        $resp = $this->nfe->tagcomb($tagCombs);
    }
    
    private function criaTagEncerrante($nItem, $combs) {
        $nBico = $combs['NRBICO'];
        $nBomba = $combs['NRBOMBA'];
        $nTanque = $combs['NRTANQUE'];
        $vEncIni = $combs['VRENCERRAINI'];
        $vEncFin = $combs['VRENCERRAFIM'];
        
        $tagEncerrante = new \stdClass();
        $tagEncerrante->item    = $nItem;
        $tagEncerrante->nBico   = $nBico;
        $tagEncerrante->nBomba  = $nBomba;
        $tagEncerrante->nTanque = $nTanque;
        $tagEncerrante->vEncIni = $vEncIni;
        $tagEncerrante->vEncFin = $vEncFin;
        
        $resp = $this->nfe->tagencerrante($tagEncerrante);
    }
    
    private function criaTagISSQN($nItem, $rowItemProd, $rowISSQN)
    {
        $cSitTrib = "";
        $this->valorTotalVSERV += $this->util->decimalToFloat($rowItemProd["VRTOTAITEMFLOAT"]);
        $vBC = $rowISSQN["BASECALCULO"];
        $this->valorTotalVBCISSQN += $this->util->decimalToFloat($rowISSQN["VRBASECALCIT"]);
        $vAliq = number_format($this->util->decimalToFloat($rowISSQN["ALIQUOTA"]), 2, ".", "");
        $vISSQN = number_format($this->util->decimalToFloat($rowISSQN["VALORIMPOSTO"]), 2, ".", "");
        $this->valorTotalVISS += $this->util->decimalToFloat($rowISSQN["VRIMPOSTOIT"]);
        $cMunFG = $this->corpoNF["CDMUNIIBGEEMIT"];
        //Verifica o codigo para classificação do serviço

        $rowClassServico = $this->transmissaoNfeFactory->consultaClassificacaoServico($rowItemProd["CDARVPROD"]);

        $cListServ = $rowClassServico["CDCLASSPROD"];
        if (!empty($rowISSQN["CDTRIBISSQN"])) {
            $cSitTrib = $rowISSQN["CDTRIBISSQN"];
        }

        $this->tags["ISSQN"] = true;
        
        $tagISSQN = new \stdClass();
        $tagISSQN->item     = $nItem;
        $tagISSQN->vBC      = $vBC;
        $tagISSQN->vAliq    = $vAiq;
        $tagISSQN->cMunFG   = $cMunFG;
        $tagISSQN->cListServ = $cListServ;
        
        $resp = $this->nfe->tagISSQN($tagISSQN);
    }

    /**
     * Imposto [ICMS]
     * @param $rowItemProd
     * @param $rowICMS
     * @param \Geral\Util\Dataset $dstImpostos
     * @return void
     */
    private function criaTagICMS($nItem, $rowItemProd, $rowICMS, $impostos, $fcp)
    {
        if ($this->nfeTools->config->regime === "1") {
            //Quando Simples nacional(201/202/203/500/900) busca na tabela o código do imposto passando os parametros filial e Numero da notafiscal e encaminha o array para a função;
            $cdImposto = $this->transmissaoNfeFactory->impostoTrib($rowItemProd["CDFILIAL"], $rowItemProd["NRLANCTONF"]);

			if ($rowItemProd["CDFISCOPERSN"] === "101") {
                //Imposto.ICMS.ICMSSN10
                $this->criaTagICMSSN101($nItem, $rowICMS, $fcp);
            } elseif ($rowItemProd["CDFISCOPERSN"] === "201") {
                //Imposto.ICMS.ICMSSN201
                $this->criaTagICMSSN201($nItem, $rowICMS, $impostos, $fcp, $cdImposto);
            } elseif ($rowItemProd["CDFISCOPERSN"] === "500") {
                //Imposto.ICMS.ICMSSN500
                $this->criaTagICMSSN500($nItem, $rowICMS, $impostos, $fcp, $cdImposto);
            } elseif ($rowItemProd["CDFISCOPERSN"] === "900") {
                //Imposto.ICMS.ICMSSN900
                $this->criaTagICMSSN900($nItem, $rowICMS, $impostos, $fcp, $cdImposto);
            } elseif (in_array($rowItemProd["CDFISCOPERSN"], array("202", "203"))) {
                //Imposto.ICMS.ICMSSN202
                $this->criaTagICMSSN202($nItem, $rowICMS, $impostos, $fcp, $rowItemProd["CDFISCOPERSN"], $cdImposto);
            } elseif (in_array($rowItemProd["CDFISCOPERSN"], array("102", "103", "300", "400"))) {
                //Imposto.ICMS.ICMSSN102
                $this->criaTagICMSSN102($nItem, $rowICMS, $rowItemProd, $fcp);
            }
        } else {
            if ($rowICMS["CST"] === "00") {
                //Imposto.ICMS.ICMS00
                $this->criaTagICMS00($nItem, $rowICMS, $fcp);
            } elseif ($rowICMS["CST"] === "10") {
                //Imposto.ICMS.ICMS10
                $this->criaTagICMS10($nItem, $rowICMS, $impostos, $fcp);
            } elseif ($rowICMS["CST"] === "20") {
                //Imposto.ICMS.ICMS20
                $this->criaTagICMS20($nItem, $rowICMS, $fcp);
            } elseif ($rowICMS["CST"] === "30") {
                //Imposto.ICMS.ICMS30
                $this->criaTagICMS30($nItem, $rowICMS, $impostos, $fcp);
            } elseif (in_array($rowICMS["CST"], array("40", "41", "50"))) {
                //Imposto.ICMS.ICMS40
                $this->criaTagICMS40($nItem, $rowICMS, $fcp);
            } elseif ($rowICMS["CST"] === "51") {
                //Imposto.ICMS.ICMS51
                $this->criaTagICMS51($nItem, $rowICMS, $fcp);
            } elseif ($rowICMS["CST"] === "60") {
                //Imposto.ICMS.ICMS60
                $this->criaTagICMS60($nItem, $rowICMS, $impostos, $fcp);
            } elseif ($rowICMS["CST"] === "70") {
                //Imposto.ICMS.ICMS70
                $this->criaTagICMS70($nItem, $rowICMS, $impostos, $fcp);
            } elseif ($rowICMS["CST"] === "90") {
                //Imposto.ICMS.ICMS90
                $this->criaTagICMS90($nItem, $rowICMS, $impostos, $fcp);
            }
        }
    }

    /**
     * Grupo CRT=1  Simples Nacional e CSOSN=101 [ICMS]
     * @param $rowICMS
     */
    private function criaTagICMSSN101($nItem, $rowICMS, $fcp = true)
    {
        //Imposto.ICMS.ICMSSN101
        $orig = $rowICMS["ORIGEMSN"];
        $CSOSN = "101";
        if (!empty($rowICMS["ALIQUOTA"])) {
            $pCredSN = number_format($this->util->decimalToFloat($rowICMS["ALIQUOTA"]), 2, ".", "");
        } else {
            $pCredSN = "0.00";
        }
        if (!empty($rowICMS["VALORIMPOSTO"])) {
            $vCredICMSSN = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
        } else {
            $vCredICMSSN = "0.00";
        }

        $tagICMSSN = new \stdClass();
        $tagICMSSN->item    = $nItem;
        $tagICMSSN->orig    = $orig;
        $tagICMSSN->CSOSN   = $CSOSN;
        $tagICMSSN->pCredSN = $pCredSN;
        $tagICMSSN->vCredICMSSN = $vCredICMSSN;
    
        $this->nfe->tagICMSSN($tagICMSSN);
    }

    /**
     * Grupo CRT=1  Simples Nacional e CSOSN=201 [ICMS]
     * @param $rowICMS
     * @param $impostos
     */
    private function criaTagICMSSN201($nItem, $rowICMS, $impostos , $fcp = true, $cdImposto)
    {
        $tagICMSSN = new \stdClass();
        $pRedBCST= "";
        $pMVAST= "";
        $orig= $rowICMS["ORIGEMSN"];
        $CSOSN= '201';
        $modBCST= $rowICMS["IDMODBCSTITNFE"];
        
        if (!empty($rowICMS["CDIMPOSTRIB"])) {
            $aux = array_search($rowICMS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            
            if($rowImpostrib){
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST= number_format($this->util->decimalToFloat($rowImpostrib["PERCENTUALRED"]), 2, ".", "");
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST= $rowImpostrib["BASECALCULO"];
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST= "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST= number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST= "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST= number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST= "0.00";
                }
                
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMSSN->vBCFCPST = $vBCST;
                    $tagICMSSN->pFCPST   = $rowImpostrib["VALORADICIONALALIQFECP"];
                    $tagICMSSN->vFCPST   = $rowImpostrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostrib["VRADICFECP"];
                }
            }
            else{
                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }
        } else {
            
            $aux = array_search($cdImposto[0]["CDIMPOSTO"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';

            if($rowImpostrib){
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST= number_format($this->util->decimalToFloat($rowImpostrib["PERCENTUALRED"]), 2, ".", "");
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST= $rowImpostrib["BASECALCULO"];
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST= "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST= number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST= "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST= number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST= "0.00";
                }
                
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMSSN->vBCFCPST = $vBCST;
                    $tagICMSSN->pFCPST   = $rowImpostrib["VALORADICIONALALIQFECP"];
                    $tagICMSSN->vFCPST   = $rowImpostrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostrib["VRADICFECP"];
                }
            } else{
                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }
        }
        if (!empty($rowICMS["ALIQUOTA"])) {
                $pCredSN = number_format($this->util->decimalToFloat($rowICMS["ALIQUOTA"]), 2, ".", "");
        } else {
            $pCredSN = "0.00";
        }
        if (!empty($rowICMS["VALORIMPOSTO"])) {
                $vCredICMSSN = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
        } else {
            $vCredICMSSN = "0.00";
        }
        
        $tagICMSSN->item        = $nItem;
        $tagICMSSN->orig        = $orig;
        $tagICMSSN->CSOSN       = $CSOSN;
        $tagICMSSN->pCredSN     = $pCredSN;
        $tagICMSSN->vCredICMSSN = $vCredICMSSN;
        $tagICMSSN->modBCST     = $modBCST;
        $tagICMSSN->pMVAST      = $pMVAST;
        $tagICMSSN->pRedBCST    = $pRedBCST;
        $tagICMSSN->vBCST       = $vBCST;
        $tagICMSSN->pICMSST     = $pICMSST;
        $tagICMSSN->vICMSST     = $vICMSST;
        
        $this->nfe->tagICMSSN($tagICMSSN);
    }

    //ICMSPart - ICMS em Operações Interestaduais - CST 10
    //$resp = $nfe->tagICMSPart($nItem, $orig, $cst, $modBC, $vBC, $pRedBC, $pICMS, $vICMS, $modBCST, $pMVAST, $pRedBCST, $vBCST, $pICMSST, $vICMSST, $pBCOp, $ufST);

    /**
     * Grupo CRT=1 Simples Nacional e CSOSN = 500 [ICMS]
     * @param $rowICMS
     * @param $impostos
     */
    private function criaTagICMSSN500($nItem, $rowICMS, $impostos, $fcp = true, $cdImposto)
    {
        $tagICMSSN = new \stdClass();
        $orig= $rowICMS["ORIGEMSN"];
        $CSOSN= "500";
        $vICMSSubstituto = $rowICMS["VALORIMPOSTO"];
        
        if (!empty($rowICMS["CDIMPOSTRIB"])) {
            
            $aux = array_search($rowICMS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            
            if (!empty($rowImpostrib["BASECALCULORETIDO"])) {
                $vBCSTRet= number_format($this->util->decimalToFloat($rowImpostrib["BASECALCULORETIDO"]), 2, ".", "");
            } else {
                $vBCSTRet= "0.00";
            }
            if (!empty($rowImpostrib["VALORIMPOSTORETIDO"])) {
                $vICMSSTRet= number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTORETIDO"]), 2, ".", "");
            } else {
                $vICMSSTRet= "0.00";
            }
            if (!empty($rowImpostrib["ALIQUOTA"])) {
                if (!empty($rowICMS["VALORIMPOSTO"])) {
                    $vICMSSubstituto = number_format($this->util->decimalToFloat($vICMSSubstituto), 2, ".", "");
                }
            } else {
                $vICMSSubstituto = '0.00';
            }
            if(!empty($rowImpostrib["ALIQUOTA"]) || !empty($rowImpostrib["VALORADICIONALALIQFECP"])) {
                $PST = $rowImpostrib["ALIQUOTA"] + $rowImpostrib["VALORADICIONALALIQFECP"];
                $PST = number_format($this->util->decimalToFloat($PST), 2, ".", "");
            }
            else {
                $PST = '0.00';
            }
            
            if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                $tagICMSSN->vBCFCPSTRet = $vBCSTRet;
                $tagICMSSN->pFCPSTRet   = $rowImpostrib["VALORADICIONALALIQFECP"];
                $tagICMSSN->vFCPSTRet   = $rowImpostrib["VALORADICIONALFECP"];
                $this->valorAdicionalFCPST += $rowImpostrib["VRADICFECP"];
            }
        } else if(isset($cdImposto[0]["CDIMPOSTO"])) {
            
                $aux = array_search($cdImposto[0]["CDIMPOSTO"], array_column($impostos, "CDIMPOSTO"));
                $rowImpostrib = $aux !== false ? $impostos[$aux] : '';

            if ($rowImpostrib) {
                
                if (!empty($rowImpostrib["BASECALCULORETIDO"])) {
                    $vBCSTRet= number_format($this->util->decimalToFloat($rowImpostrib["BASECALCULORETIDO"]), 2, ".", "");
                } else {
                    $vBCSTRet= "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTORETIDO"])) {
                    $vICMSSTRet= number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTORETIDO"]), 2, ".", "");
                } else {
                    $vICMSSTRet= "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    if (!empty($rowICMS["VALORIMPOSTO"])) {
                        $vICMSSubstituto = number_format($this->util->decimalToFloat($vICMSSubstituto), 2, ".", "");
                    }
                } else {
                    $vICMSSubstituto = '0.00';
                }
                if(!empty($rowImpostrib["ALIQUOTA"]) || !empty($rowImpostrib["VALORADICIONALALIQFECP"])) {
                    $PST = $rowImpostrib["ALIQUOTA"] + $rowImpostrib["VALORADICIONALALIQFECP"];
                    $PST = number_format($this->util->decimalToFloat($PST), 2, ".", "");
                }
                else {
                    $PST = '0.00';
                }
                
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMSSN->vBCFCPSTRet = $vBCSTRet;
                    $tagICMSSN->pFCPSTRet   = $rowImpostrib["VALORADICIONALALIQFECP"];
                    $tagICMSSN->vFCPSTRet   = $rowImpostrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostrib["VRADICFECP"];
                }
                
            } else {

                $vBCSTRet= '0.00';
                $vICMSSTRet= '0.00';
                $vICMSSubstituto = '0.00';
                $PST = '0.00'; 
            }
        }
        else {
            $vBCSTRet= '0.00';
            $vICMSSTRet= '0.00';
            $vICMSSubstituto = '0.00';
            $PST = '0.00'; 
        }
        /*
                             if($this->corpoNF['IDCONSFINAL'] == 'S')
                                 adqImpostos.Filtered := False;
                                 adqImpostos.Filter   := '(CDIMPOSTONFE = ''00'')';
                                 adqImpostos.Filtered := True;
                                 PRedBCEfet := adqImpostosPERCENTUALRED.AsString;
                                 VBCEfet    := adqImpostosBASECALCULO.AsString;
                                 PICMSEfet  := adqImpostosALIQUOTAV400.AsString;
                                 VICMSEfet  := adqImpostosVALORIMPOSTOV400.AsString;
                              }
        */
        $tagICMSSN->item    = $nItem;
        $tagICMSSN->orig    = $orig;
        $tagICMSSN->CSOSN   = $CSOSN;
        $tagICMSSN->vBCSTRet = $vBCSTRet;
        $tagICMSSN->pST      = $PST;
        $tagICMSSN->vICMSSubstituto = $vICMSSubstituto;
        $tagICMSSN->vICMSSTRet = $vICMSSTRet;
    
        $this->nfe->tagICMSSN($tagICMSSN);
    }

    /**
     * Grupo CRT=1 Simples Nacional e CSOSN=900 [ICMS][N10h]
     * @param $rowICMS
     * @param $impostos
     */
    private function criaTagICMSSN900($nItem, $rowICMS, $impostos, $fcp = true, $cdImposto)
    {
        $tagICMSSN = new \stdClass();
        
        $pMVAST = "";
        $pRedBCST = "";
        $pRedBC = "";
        $orig = $rowICMS["ORIGEMSN"];
        $CSOSN = '900';
        $modBC = $rowICMS["IDMODBCITNFE"];
        if ($rowICMS["CDFISCALIT"] === "1") {
            if (!empty($rowICMS["BASECALCULO"])) {
                $vBC = number_format($this->util->decimalToFloat($rowICMS["BASECALCULO"]), 2, ".", "");
            } else {
                $vBC = "0.00";
            }
            if ($this->util->decimalToFloat($rowICMS["VRPERREDBCIT"]) > 0) {
                $pRedBC = number_format($this->util->decimalToFloat($rowICMS["PERCENTUALRED"]), 2, ".", "");
            }
            if (!empty($rowICMS["ALIQUOTA"])) {
                $pICMS = number_format($this->util->decimalToFloat($rowICMS["ALIQUOTA"]), 2, ".", "");
            } else {
                $pICMS = "0.00";
            }
            if (!empty($rowICMS["VALORIMPOSTO"])) {
                $vICMS = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
            } else {
                $vICMS = "0.00";
            }
            $this->valorTotalBC += $this->util->decimalToFloat($rowICMS["VRBASECALCIT"]);
            $this->valorTotalICMS += $this->util->decimalToFloat($rowICMS["VRIMPOSTOIT"]);
            
        } else {
            $vBC = '0.00';
            $pICMS = '0.00';
            $vICMS = '0.00';
        }
        $modBCST = $rowICMS["IDMODBCSTITNFE"];
        if (!empty($rowICMS["CDIMPOSTRIB"])) {
            $aux = array_search($rowICMS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            if($rowImpostrib){
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST = $rowImpostrib["PERCENTUALRED"];
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST = $rowImpostrib["BASECALCULO"];
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST = "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST = number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST = "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST = "0.00";
                }
                
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMSSN->vBCFCPST = $vBCST;
                    $tagICMSSN->pFCPST   = $rowImpostrib["VALORADICIONALALIQFECP"];
                    $tagICMSSN->vFCPST   = $rowImpostrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostrib["VRADICFECP"];
                }
            }
            else{

                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }

        } else if(isset($cdImposto[0]["CDIMPOSTO"])){
            // Busca o imposto no array de (impostos) filtrando pelo código 99
            $aux = array_search($cdImposto[0]["CDIMPOSTO"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            // se retornar os impostos filtrados, os calculos serão realizados baseados no codigo do imposto passado 
            if ($rowImpostrib) {
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST= number_format($this->util->decimalToFloat($rowImpostrib["PERCENTUALRED"]), 2, ".", "");
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST= $rowImpostrib["BASECALCULO"];
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST= "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST= number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST= "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST= number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST= "0.00";
                }
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMSSN->vBCFCPST = $vBCST;
                    $tagICMSSN->pFCPST   = $rowImpostrib["VALORADICIONALALIQFECP"];
                    $tagICMSSN->vFCPST   = $rowImpostrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostrib["VRADICFECP"];
                }
            // Se não valor da Base de calculo da subistituição tributaria, parametro ICMS-ST, valor do ICMS-ST    
            // recebem 0.00
            } else {
                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }
        } else {
            $vBCST = '0.00';
            $pICMSST = '0.00';
            $vICMSST = '0.00';
        }
        
        if (!empty($rowICMS["BASECALCULORETIDO"])) {
            $vBCSTRet = number_format($this->util->decimalToFloat($rowICMS["BASECALCULORETIDO"]), 2, ".", "");
        } else {
            $vBCSTRet = "0.00";
        }
        if (!empty($rowICMS["VALORIMPOSTORETIDO"])) {
            $vICMSSTRet = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTORETIDO"]), 2, ".", "");
        } else {
            $vICMSSTRet = "0.00";
        }
        if (!empty($rowICMS["ALIQUOTA"])) {
            $pCredSN = number_format($this->util->decimalToFloat($rowICMS["ALIQUOTA"]), 2, ".", "");
        } else {
            $pCredSN = "0.00";
        }
        if (!empty($rowICMS["VALORIMPOSTO"])) {
            $vCredICMSSN = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
        } else {
            $vCredICMSSN = "0.00";
        }
        
        
        $tagICMSSN->item = $nItem;
        $tagICMSSN->orig = $orig;
        $tagICMSSN->CSOSN = $CSOSN;
        $tagICMSSN->modBC = $modBC;
        $tagICMSSN->vBC = $vBC;
        $tagICMSSN->pRedBC = $pRedBC;
        $tagICMSSN->pICMS = $pICMS;
        $tagICMSSN->vICMS = $vICMS;
        $tagICMSSN->pCredSN = $pCredSN;
        $tagICMSSN->vCredICMSSN = $vCredICMSSN;
        $tagICMSSN->modBCST = $modBCST;
        $tagICMSSN->pMVAST = $pMVAST;
        $tagICMSSN->pRedBCST = $pRedBCST;
        $tagICMSSN->vBCST = $vBCST;
        $tagICMSSN->pICMSST = $pICMSST;
        $tagICMSSN->vICMSST = $vICMSST;
        $tagICMSSN->vBCSTRet = $vBCSTRet;
        $tagICMSSN->vICMSSTRet = $vICMSSTRet;
        
        $this->nfe->tagICMSSN($tagICMSSN);
    }
    
    private function criaTagICMSUfDest($nItem, $rowICMS, $fcp = false) {
        if($rowICMS["VALORBASECALCDESTINO"]     != null
          || $rowICMS["VALORALIQUOTAFCP"]       != null
          || $rowICMS["VALORALIQINTEDESTINO"]   != null
          || $rowICMS["VALORPERCENTUALPARTILHA"]!= null
          || $rowICMS["VALORIMPOSTOFCP"]        != null
          || $rowICMS["VALORIMPOPARTDESTINO"]   != null
          || $rowICMS["VALORIMPOPARTREMETENTE"] != null) {
              
            $tagICMSUFDest = new \stdClass();
            $tagICMSUFDest->item = $nItem;
            
            // @fcp -> True para o caso do destinatario ser consumidor final e a 
            // operação interestadual o que obriga que o FCP seja informado
            // na tag ICMSUFDEST
            
            if($rowICMS["VALORBASECALCDESTINO"]     != null) {
                $tagICMSUFDest->vBCUFDest    = $rowICMS["VALORBASECALCDESTINO"];
                $tagICMSUFDest->vBCFCPUFDest = $rowICMS["VALORBASECALCDESTINO"];   
            }
            if($rowICMS["VALORALIQUOTAFCP"]         != null) {
                    $tagICMSUFDest->pFCPUFDest   = $rowICMS["VALORALIQUOTAFCP"];
            }
            if($rowICMS["VALORALIQINTEDESTINO"]     != null) {
                $tagICMSUFDest->pICMSUFDest  = $rowICMS["VALORALIQINTEDESTINO"];
            }
            if($rowICMS["VALORALIQINTERESTADUAL"]   != null) {
                $tagICMSUFDest->pICMSInter   = number_format($rowICMS["VALORALIQINTERESTADUAL"], 2, ".", "");
            }
            if($rowICMS["VALORPERCENTUALPARTILHA"]  != null) {
                $tagICMSUFDest->pICMSInterPart  = $rowICMS["VALORPERCENTUALPARTILHA"];
            }
            if($rowICMS["VALORIMPOSTOFCP"]          != null) {
                $tagICMSUFDest->vFCPUFDest      = $rowICMS["VALORIMPOSTOFCP"];
                $this->valorAdicionalFCPDest   += $rowICMS["VALORIMPOSTOFCP"];
            }
            if($rowICMS["VALORIMPOPARTDESTINO"]     != null) {
                $tagICMSUFDest->vICMSUFDest      = $rowICMS["VALORIMPOPARTDESTINO"];
                $this->valorTotalPartDest       += $rowICMS["VRIMPOPARTDEST"];
            }
            if($rowICMS["VALORIMPOPARTREMETENTE"]   != null) {
                $tagICMSUFDest->vICMSUFRemet    = $rowICMS["VALORIMPOPARTREMETENTE"];
                $this->valorTotalPartRemet       += $rowICMS["VRIMPOPARTREME"];
            }
            
            $resp = $this->nfe->tagICMSUFDEST($tagICMSUFDest);      
        }
    }


    /**
     * Grupo CRT=1  Simples Nacional e CSOSN=202 ou 203 [ICMS]
     * @param $rowICMS
     * @param \Geral\Util\Dataset $dstImpostos
     */
    private function criaTagICMSSN202($nItem, $rowICMS, $impostos, $fcp = true, $rowItemProdCSOSN, $cdImposto)
    {
        $tagICMSSN = new \stdClass();
        
        $pRedBCST = "";
        $pMVAST = "";
        $vBCST = "";
        $pICMSST = "";
        $vICMSST = "";
        $orig = $rowICMS["ORIGEMSN"];
        $CSOSN = $rowItemProdCSOSN;
        $modBCST = $rowICMS["IDMODBCSTITNFE"];
        if (!empty($rowICMS["CDIMPOSTRIB"])) {
            $aux = array_search($rowICMS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            if($rowImpostrib){
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST = number_format($this->util->decimalToFloat($rowImpostrib["PERCENTUALRED"]), 2, ".", "");
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST = $rowImpostrib["BASECALCULO"];
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST = "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST = number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST = "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST = "0.00";
                }
                
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMSSN->vBCFCPST = $vBCST;
                    $tagICMSSN->pFCPST   = $rowImpostrib["VALORADICIONALALIQFECP"];
                    $tagICMSSN->vFCPST   = $rowImpostrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostrib["VRADICFECP"];
                }
            }
            else{
                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }
        } else {
            // Busca o imposto no array de (impostos) filtrando pelo código 99
            $aux = array_search($cdImposto[0]["CDIMPOSTO"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            // se retornar os impostos filtrados, os calculos serão realizados baseados no codigo do imposto passado 
            if ($rowImpostrib) {
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST= number_format($this->util->decimalToFloat($rowImpostrib["PERCENTUALRED"]), 2, ".", "");
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST= $rowImpostrib["BASECALCULO"];
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST= "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST= number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST= "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST= number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST= "0.00";
                }
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMSSN->vBCFCPST = $vBCST;
                    $tagICMSSN->pFCPST   = $rowImpostrib["VALORADICIONALALIQFECP"];
                    $tagICMSSN->vFCPST   = $rowImpostrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostrib["VRADICFECP"];
                }
            // Se não valor da Base de calculo da subistituição tributaria, parametro ICMS-ST, valor do ICMS-ST    
            // recebem 0.00
            } else{
                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }
        }

        $tagICMSSN->item = $nItem;
        $tagICMSSN->orig = $orig;
        $tagICMSSN->CSOSN = $CSOSN;
        $tagICMSSN->modBCST = $modBCST;
        $tagICMSSN->pMVAST = $pMVAST;
        $tagICMSSN->pRedBCST = $pRedBCST;
        $tagICMSSN->vBCST = $vBCST;
        $tagICMSSN->pICMSST = $pICMSST;
        $tagICMSSN->vICMSST = $vICMSST;

        $this->nfe->tagICMSSN($tagICMSSN);
    }

    /**
     * Grupo CRT=1 Simples Nacional e CSOSN=102, 103,300 ou 400 [ICMS] [N10d]
     * @param $rowICMS
     * @param $rowItemProd
     */
    private function criaTagICMSSN102($nItem, $rowICMS, $rowItemProd, $fcp = true)
    {
        $tagICMSSN = new \stdClass();
      
        $orig = $rowICMS["ORIGEMSN"];
        $CSOSN = $rowItemProd["CDFISCOPERSN"];

        $tagICMSSN->item  = $nItem;
        $tagICMSSN->orig  = $orig;
        $tagICMSSN->CSOSN = $CSOSN;

        $this->nfe->tagICMSSN($tagICMSSN);
    }

    /**
     * CST 00 TRIBUTADO INTEGRALMENTE [ICMS] [N02]
     * @param $rowICMS
     * @return array
     */
     // VERIFICAR ICMS00
    private function criaTagICMS00($nItem, $rowICMS, $fcp = true)
    {
        $tagICMS = new \stdClass();
        
        $orig = $rowICMS["ORIGEM"];
        $CST = $rowICMS["CST"];
        $modBC = $rowICMS["IDMODBCITNFE"];
        if ($rowICMS["CDFISCALIT"] === "1") {
            $vBC = number_format($this->util->decimalToFloat($rowICMS["BASECALCULO"]), 2, ".", "");
            $pICMS = number_format($this->util->decimalToFloat($rowICMS["ALIQUOTA"]), 2, ".", "");
            $vICMS = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
            $this->valorTotalBC += $rowICMS["VRBASECALCIT"];
            $this->valorTotalICMS += $rowICMS["VRIMPOSTOIT"];
            
            if(!empty($rowICMS["VRADICALIQFECP"]) && $fcp === true) {
                $tagICMS->pFCP   = $rowICMS["VALORADICIONALALIQFECP"];
                $tagICMS->vFCP   = $rowICMS["VALORADICIONALFECP"];
                $this->valorAdicionalFCP += $rowICMS["VRADICFECP"];
            }
        } else {
            $vBC = "0.00";
            $pICMS = "0.00";
            $vICMS = "0.00";
        }
        
        
        $tagICMS->item = $nItem;
        $tagICMS->orig = $orig;
        $tagICMS->CST = $CST;
        $tagICMS->modBC = $modBC;
        $tagICMS->vBC = $vBC;
        $tagICMS->pICMS = $pICMS;
        $tagICMS->vICMS = $vICMS;
        
        $resp = $this->nfe->tagICMS($tagICMS);
    }

    /**
     * CST 010 TRIBUTADO E COM COBRANCAO DE ICMS POR SUBSTUICAO TRIBUTARIA [ICMS] [N03]
     * @param $rowICMS
     * @param $impostos
     * @return array
     */
    private function criaTagICMS10($nItem, $rowICMS, $impostos, $fcp = true)
    {
        $tagICMS = new \stdClass();
        
        $pRedBCST = "";
        $pMVAST = "";
        $orig = $rowICMS["ORIGEM"];
        $CST = $rowICMS["CST"];
        $modBC = $rowICMS["IDMODBCITNFE"];
        if ($rowICMS["CDFISCALIT"] === "1") {
            $vBC = number_format($this->util->decimalToFloat($rowICMS["BASECALCULO"]), 2, ".", "");
            $pICMS = number_format($this->util->decimalToFloat($rowICMS["ALIQUOTA"]), 2, ".", "");
            $vICMS = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
            $this->valorTotalBC += $this->util->decimalToFloat($rowICMS["VRBASECALCIT"]);
            $this->valorTotalICMS += $this->util->decimalToFloat($rowICMS["VRIMPOSTOIT"]);
            
            if(!empty($rowICMS["VRADICALIQFECP"]) && $fcp === true) {
                $tagICMS->vBCFCP = $vBC;
                $tagICMS->pFCP   = $rowICMS["VALORADICIONALALIQFECP"];
                $tagICMS->vFCP   = $rowICMS["VALORADICIONALFECP"];
                $this->valorAdicionalFCP += $rowICMS["VRADICFECP"];
            }
            
        } else {
            $vBC = "0.00";
            $pICMS = "0.00";
            $vICMS = "0.00";
        }
        $modBCST = $rowICMS["IDMODBCSTITNFE"];
        if (!empty($rowICMS["CDIMPOSTRIB"])) {
            $aux = array_search($rowICMS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            if($rowImpostrib){
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST = $rowImpostrib["PERCENTUALRED"];
                } else {
                    $pMVAST = $rowImpostrib["PERCENTUALRED"];
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST = number_format($this->util->decimalToFloat($rowImpostrib["BASECALCULO"]), 2, ".", "");
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST = "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST = number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST = "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST = "0.00";
                }
                
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMS->vBCFCPST = $vBCST;
                    $tagICMS->pFCPST   = $rowImpostrib["VALORADICIONALALIQFECP"];
                    $tagICMS->vFCPST   = $rowImpostrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostrib["VRADICFECP"];
                }
            }
            else{
                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }

        } else {
            $vBCST = '0.00';
            $pICMSST = '0.00';
            $vICMSST = '0.00';
        }
        
        $tagICMS->item = $nItem;
        $tagICMS->orig = $orig;
        $tagICMS->CST = $CST;
        $tagICMS->modBC = $modBC;
        $tagICMS->vBC = $vBC;
        $tagICMS->pICMS = $pICMS;
        $tagICMS->vICMS = $vICMS;
        $tagICMS->modBCST = $modBCST;
        $tagICMS->pMVAST = $pMVAST;
        $tagICMS->pRedBCST = $pRedBCST;
        $tagICMS->vBCST = $vBCST;
        $tagICMS->pICMSST = $pICMSST;
        $tagICMS->vICMSST = $vICMSST;

        $this->nfe->tagICMS($tagICMS);
    }

    /**
     * CST 020 COM REDUCAO DE BASE DE CALCULO [ICMS] [N04]
     * @param $rowICMS
     */
    private function criaTagICMS20($nItem, $rowICMS, $fcp = true)
    {
        $tagICMS = new \stdClass();
        
        $orig = $rowICMS["ORIGEM"];
        $CST = $rowICMS["CST"];
        $modBC = $rowICMS["IDMODBCITNFE"];
        if ($this->util->decimalToFloat($rowICMS["VRPERREDBCIT"]) > 0) {
            $pRedBC = number_format($this->util->decimalToFloat($rowICMS["PERCENTUALRED"]), 2, ".", "");
        } else {
            $pRedBC = "0.00";
        }
        if ($rowICMS["CDFISCALIT"] === "1") {
            $vBC = $rowICMS["BASECALCULO"];
            $pICMS = $rowICMS["ALIQUOTA"];
            $vICMS = $rowICMS["VALORIMPOSTO"];
            $this->valorTotalBC += $this->util->decimalToFloat($rowICMS["VRBASECALCIT"]);
            $this->valorTotalICMS += $this->util->decimalToFloat($rowICMS["VRIMPOSTOIT"]);
            
            if(!empty($rowICMS["VRADICALIQFECP"]) && $fcp === true) {
                $tagICMS->vBCFCP = $vBC;
                $tagICMS->pFCP   = $rowICMS["VALORADICIONALALIQFECP"];
                $tagICMS->vFCP   = $rowICMS["VALORADICIONALFECP"];
                $this->valorAdicionalFCP += $rowICMS["VRADICFECP"];
            }
        } else {
            $vBC = '0.00';
            $pICMS = '0.00';
            $vICMS = '0.00';
        }

        $vICMSDeson = '';
        $motDesICMS = '';
        if($rowICMS['IDIMPDESONERAIT'] == 'S'){
            $this->valorTotalICMSDeson += $rowICMS['VRIMPOSTOIT'];
            $vICMSDeson = $vICMS;
            $motDesICMS = $rowICMS['IDMOTDESONERAIT'];
        }
        
        $tagICMS->item = $nItem;
        $tagICMS->orig = $orig;
        $tagICMS->CST = $CST;
        $tagICMS->modBC = $modBC;
        $tagICMS->pRedBC = $pRedBC;
        $tagICMS->vBC = $vBC;
        $tagICMS->pICMS = $pICMS;
        $tagICMS->vICMS = $vICMS;
        $tagICMS->vICMSDeson = $vICMSDeson;
        $tagICMS->motDesICMS = $motDesICMS;
        
        $this->nfe->tagICMS($tagICMS);
    }

    /**
     * CST 030 ISENTA OU NAO TRIBUTADO E COM COBRANCA DO ICMS POR ST [ICMS] [N05]
     * @param $rowICMS
     * @param $impostos
     */
    private function criaTagICMS30($nItem, $rowICMS, $impostos, $fcp = true)
    {
        $tagICMS = new \stdClass();
        
        $pRedBCST = "";
        $pMVAST = "";
        $orig = $rowICMS["ORIGEM"];
        $CST = $rowICMS["CST"];
        $modBCST = $rowICMS["IDMODBCSTITNFE"];

        $vICMS = $rowICMS['VALORIMPOSTO'];
        $vICMSDeson = '';
        $motDesICMS = '';
        if($rowICMS['IDIMPDESONERAIT'] == 'S'){
            $this->valorTotalICMSDeson += $rowICMS['VRIMPOSTOIT'];
            $vICMSDeson = $vICMS;
            $motDesICMS = $rowICMS['IDMOTDESONERAIT'];
        }

        if (!empty($rowICMS["CDIMPOSTRIB"])) {
            $aux = array_search($rowICMS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            if($rowImpostrib){
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST = $rowImpostrib["PERCENTUALRED"];
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST = number_format($this->util->decimalToFloat($rowImpostrib["BASECALCULO"]), 2, ".", "");
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST = "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST = number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST = "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST = "0.00";
                }
                
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMS->vBCFCPST = $vBCST;
                    $tagICMS->pFCPST   = $rowImpostoTrib["VALORADICIONALALIQFECP"];
                    $tagICMS->vFCPST   = $rowImpostoTrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostoTrib["VRADICFECP"];
                }
            }
            else{
                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }
        } else {
            $vBCST = '0.00';
            $pICMSST = '0.00';
            $vICMSST = '0.00';
        }
            
        $tagICMS->item = $nItem;
        $tagICMS->orig = $orig;
        $tagICMS->CST = $CST;
        $tagICMS->vICMS = $vICMS;
        $tagICMS->vICMSDeson = $vICMSDeson;
        $tagICMS->motDesICMS = $motDesICMS;
        $tagICMS->modBCST = $modBCST;
        $tagICMS->pMVAST = $pMVAST;
        $tagICMS->pRedBCST = $pRedBCST;
        $tagICMS->vBCST = $vBCST;
        $tagICMS->pICMSST = $pICMSST;
        $tagICMS->vICMSST = $vICMSST;
            
        $this->nfe->tagICMS($tagICMS);
    }

    /**
     * Grupo de Tributação do ICMS 40, 41 ou 50 [ICMS] [N06]
     * @param $rowICMS
     */
    private function criaTagICMS40($nItem, $rowICMS, $fcp = true)
    {
        $motDesICMS = "";
        $vICMS = "";
        $orig = $rowICMS["ORIGEM"];
        $CST = $rowICMS["CST"];
        $vICMSDeson = '';
        $motDesICMS = '';
        if ($rowICMS["IDIMPDESONERAIT"] === "S") {
            $vICMS = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
            $this->valorTotalICMSDeson += $rowICMS['VRIMPOSTOIT'];
            $vICMSDeson = $vICMS;
            $motDesICMS = $rowICMS["IDMOTDESONERAIT"];
        }
        
        $tagICMS = new \stdClass();
        $tagICMS->item = $nItem;
        $tagICMS->orig = $orig;
        $tagICMS->CST  = $CST;
        $tagICMS->vICMS = $vICMS;
        $tagICMS->vICMSDeson = $vICMSDeson;
        $tagICMS->motDesICMS = $motDesICMS;

        $this->nfe->tagICMS($tagICMS);
    }

    /**
     * Grupo de Tributação do ICMS = 51 [ICMS]
     * @param $rowICMS
     */
    private function criaTagICMS51($nItem, $rowICMS, $fcp = true)
    {
        $tagICMS = new \stdClass();
        
        $orig = $rowICMS["ORIGEM"];
        $CST = $rowICMS["CST"];
        $modBC = $rowICMS["IDMODBCITNFE"];
        if ($rowICMS["CDFISCALIT"] === "1") {
            $vBC = number_format($this->util->decimalToFloat($rowICMS["BASECALCULO"]), 2, ".", "");
            $pICMS = number_format($this->util->decimalToFloat($rowICMS["ALIQUOTA"]), 2, ".", "");
            $vICMS = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
            
            if(!empty($rowICMS["VRADICALIQFECP"]) && $fcp === true) {
                $tagICMS->vBCFCP = $vBC;
                $tagICMS->pFCP   = $rowICMS["VALORADICIONALALIQFECP"];
                $tagICMS->vFCP   = $rowICMS["VALORADICIONALFECP"];
                $this->valorAdicionalFCP += $rowICMS["VRADICFECP"];
            }
            if(!empty($rowICMS["PERCENTUALRED"])) {
                $pRedBC = number_format($this->util->decimalToFloat($rowICMS["PERCENTUALRED"]), 2, ".", "");
            } else {
                $pRedBC = "0.00";
            }
            if(!empty($rowICMS["VALORIMPOSTOOPERACAO"])) {
                $vICMSOp = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTOOPERACAO"]), 2, ".", "");
            } else {
                $vICMSOp = "0.00";
            }
            if(!empty($rowICMS["VALORPERCENTDIFERE"])) {
                $pDif = number_format($this->util->decimalToFloat($rowICMS["VALORPERCENTDIFERE"]), 2, ".", "");
            } else {
                $pDif = "0.00";
            }
            if(!empty($rowICMS["VALORIMPOSTODIFERE"])) {
                $vICMSDif = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTODIFERE"]), 2, ".", "");
            } else {
                $vICMSDif = "0.00";
            }
            
        } else {
            $vBC = '0.00';
            $pICMS = '0.00';
            $vICMS = '0.00';
            $pRedBC = "0.00";
            $vICMSOp = "0.00";
            $vICMSDif = "0.00";
            $pDif = "0.00";
        }
        $tagICMS->item = $nItem;
        $tagICMS->orig = $orig;
        $tagICMS->CST = $CST;
        $tagICMS->modBC = $modBC;
        $tagICMS->pRedBC = $pRedBC;
        $tagICMS->vBC = $vBC;
        $tagICMS->pICMS = $pICMS;
        $tagICMS->vICMSOp = $vICMSOp;
        $tagICMS->pDif = $pDif;
        $tagICMS->vICMSDif = $vICMSDif;
        $tagICMS->vICMS = $vICMS;
        
        $this->nfe->tagICMS($tagICMS);
    }

    /**
     * Grupo de Tributação do ICMS = 60 [ICMS] [N08]
     * @param $rowICMS
     * @param $impostos
     */
    private function criaTagICMS60($nItem, $rowICMS, $impostos, $fcp = true)
    {
        $tagICMS = new \stdClass();
        
        $vBCSTRet = "";
        $vICMSSTRet = "";
        $orig = $rowICMS["ORIGEM"];
        $CST = $rowICMS["CST"];
        
        if (!empty($rowICMS["CDIMPOSTRIB"])) {
            $aux = array_search($rowICMS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            if (!empty($rowImpostrib["BASECALCULO"])) {
                $vBCSTRet = number_format($this->util->decimalToFloat($rowImpostrib["BASECALCULO"]), 2, ".", "");
            }
            if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                $vICMSSTRet = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
            } else {
                $vBCSTRet = '0.00';
                $vICMSSTRet = '0.00';
            }
            if (!empty($rowImpostrib["ALIQUOTA"])) {
                if (!empty($rowICMS["VALORIMPOSTO"])) {
                    $vICMSSubstituto = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
                }
            } else {
                $vICMSSubstituto = '0.00';
            }
            if(!empty($rowImpostrib["ALIQUOTA"]) || !empty($rowImpostrib["VALORADICIONALALIQFECP"])) {
                $PST = $rowImpostrib["ALIQUOTA"] + $rowImpostrib["VALORADICIONALALIQFECP"];
                $PST = number_format($this->util->decimalToFloat($PST), 2, ".", "");
            }
            else {
                $PST = '0.00';
            }
            
            if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                $tagICMS->vBCFCPSTRet = $vBCSTRet;
                $tagICMS->pFCPSTRet   = $rowImpostrib["VALORADICIONALALIQFECP"];
                $tagICMS->vFCPSTRet   = $rowImpostrib["VALORADICIONALFECP"];
                $this->valorAdicionalFCPSTRet += $rowImpostrib["VRADICFECP"];
            }
            
        }

        $tagICMS->item = $nItem;
        $tagICMS->orig = $orig;
        $tagICMS->CST  = $CST;
        $tagICMS->vBCSTRet   = $vBCSTRet;
        $tagICMS->pST = $PST;
        $tagICMS->vICMSSubstituto = $vICMSSubstituto;
        $tagICMS->vICMSSTRet = $vICMSSTRet;

        $this->nfe->tagICMS($tagICMS);
    }

    /**
     * Grupo de Tributação do ICMS = 70 [ICMS] [N09]
     * @param $rowICMS
     * @param $impostos
     */
    private function criaTagICMS70($nItem, $rowICMS, $impostos, $fcp = true)
    {
        $tagICMS = new \stdClass();
        
        $pRedBCST = "";
        $pMVAST = "";
        $orig = $rowICMS["ORIGEM"];
        $CST = $rowICMS["CST"];
        $modBC = $rowICMS["IDMODBCITNFE"];
        $pRedBC = $rowICMS["PERCENTUALRED"];
        
        if ($rowICMS["CDFISCALIT"] === "1") {
            $vBC = number_format($this->util->decimalToFloat($rowICMS["BASECALCULO"]), 2, ".", "");
            $pICMS = number_format($this->util->decimalToFloat($rowICMS["ALIQUOTA"]), 2, ".", "");
            $vICMS = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
            $this->valorTotalBC   += $this->util->decimalToFloat($rowICMS["VRBASECALCIT"]);
            $this->valorTotalICMS += $this->util->decimalToFloat($rowICMS["VRIMPOSTOIT"]);
            
            if(!empty($rowICMS["VRADICALIQFECP"]) && $fcp === true) {
                $tagICMS->vBCFCP = $vBC;
                $tagICMS->pFCP   = $rowICMS["VALORADICIONALALIQFECP"];
                $tagICMS->vFCP   = $rowICMS["VALORADICIONALFECP"];
                $this->valorAdicionalFCP += $rowICMS["VRADICFECP"];
            }
            
        } else {
            $vBC = "0.00";
            $pICMS = "0.00";
            $vICMS = "0.00";
        }

        $vICMSDeson = '';
        $motDesICMS = '';
        if($rowICMS['IDIMPDESONERAIT'] == 'S'){
            $this->valorTotalICMSDeson += $rowICMS['VRIMPOSTOIT'];
            $vICMSDeson = $vICMS;
            $motDesICMS = $rowICMS['IDMOTDESONERAIT'];
        }

        $modBCST = $rowICMS["IDMODBCSTITNFE"];
        if (!empty($rowICMS["CDIMPOSTRIB"])) {
            $aux = array_search($rowICMS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            if($rowImpostrib){
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST = $rowImpostrib["PERCENTUALRED"];
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST = number_format($this->util->decimalToFloat($rowImpostrib["BASECALCULO"]), 2, ".", "");
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST = "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST = number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST = "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST = "0.00";
                }
                
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMS->vBCFCPST = $vBCST;
                    $tagICMS->pFCPST   = $rowImpostoTrib["VALORADICIONALALIQFECP"];
                    $tagICMS->vFCPST   = $rowImpostoTrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostoTrib["VRADICFECP"];
                }
            }
            else{
                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }
        } else {
            $vBCST = '0.00';
            $pICMSST = '0.00';
            $vICMSST = '0.00';
        }
        
        $tagICMS->item = $nItem;
        $tagICMS->orig = $orig;
        $tagICMS->CST = $CST;
        $tagICMS->modBC = $modBC;
        $tagICMS->pRedBC = $pRedBC;
        $tagICMS->vBC = $vBC;
        $tagICMS->pICMS = $pICMS;
        $tagICMS->vICMS = $vICMS;
        $tagICMS->vICMSDeson = $vICMSDeson;
        $tagICMS->motDesICMS = $motDesICMS;
        $tagICMS->modBCST = $modBCST;
        $tagICMS->pMVAST = $pMVAST;
        $tagICMS->pRedBCST = $pRedBCST;
        $tagICMS->vBCST = $vBCST;
        $tagICMS->pICMSST = $pICMSST;
        $tagICMS->vICMSST = $vICMSST;
        
        $this->nfe->tagICMS($tagICMS);
    }

    /**
     * Grupo de Tributação do ICMS 90 Outros [ICMS] [N10]
     * @param $rowICMS
     * @param $impostos
     * @return array
     */
    private function criaTagICMS90($nItem, $rowICMS, $impostos, $fcp = true)
    {
        $tagICMS = new \stdClass();
        
        $pRedBC = "";
        $pRedBCST = "";
        $pMVAST = "";
        $orig = $rowICMS["ORIGEM"];
        $CST = $rowICMS["CST"];
        $modBC = $rowICMS["IDMODBCITNFE"];
        if ($rowICMS["CDFISCALIT"] === "1") {
            $vBC = number_format($this->util->decimalToFloat($rowICMS["BASECALCULO"]), 2, ".", "");
            $pICMS = number_format($this->util->decimalToFloat($rowICMS["ALIQUOTA"]), 2, ".", "");
            $vICMS = number_format($this->util->decimalToFloat($rowICMS["VALORIMPOSTO"]), 2, ".", "");
            $this->valorTotalBC += $this->util->decimalToFloat($rowICMS["VRBASECALCIT"]);
            $this->valorTotalICMS += $this->util->decimalToFloat($rowICMS["VRIMPOSTOIT"]);
            
            if(!empty($rowICMS["VRADICALIQFECP"]) && $fcp === true) {
                $tagICMS->vBCFCP = $vBC;
                $tagICMS->pFCP   = $rowICMS["VALORADICIONALALIQFECP"];
                $tagICMS->vFCP   = $rowICMS["VALORADICIONALFECP"];
                $this->valorAdicionalFCP += $rowICMS["VRADICFECP"];
            }
        } else {
            $vBC = "0.00";
            $pICMS = "0.00";
            $vICMS = "0.00";
        }

        $vICMSDeson = '';
        $motDesICMS = '';
        if($rowICMS['IDIMPDESONERAIT'] == 'S'){
            $this->valorTotalICMSDeson += $rowICMS['VRIMPOSTOIT'];
            $vICMSDeson = $vICMS;
            $motDesICMS = $rowICMS['IDMOTDESONERAIT'];
        }

        $modBCST = $rowICMS["IDMODBCSTITNFE"];
        if (!empty($rowICMS["CDIMPOSTRIB"])) {
            $aux = array_search($rowICMS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTO"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            if($rowImpostrib){
                if ($this->util->decimalToFloat($rowImpostrib["VRPERREDBCIT"]) > 0) {
                    $pMVAST = $rowImpostrib["PERCENTUALRED"];
                }
                if (!empty($rowImpostrib["BASECALCULO"])) {
                    $vBCST = number_format($this->util->decimalToFloat($rowImpostrib["BASECALCULO"]), 2, ".", "");
                    $this->valorTotalBCST += $this->util->decimalToFloat($rowImpostrib["VRBASECALCIT"]);
                } else {
                    $vBCST = "0.00";
                }
                if (!empty($rowImpostrib["ALIQUOTA"])) {
                    $pICMSST = number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                } else {
                    $pICMSST = "0.00";
                }
                if (!empty($rowImpostrib["VALORIMPOSTO"])) {
                    $vICMSST = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                    $this->valorTotalICMSST += $this->util->decimalToFloat($rowImpostrib["VRIMPOSTOIT"]);
                } else {
                    $vICMSST = "0.00";
                }
                
                if(!empty($rowImpostrib["VRADICALIQFECP"]) && $fcp === true) {
                    $tagICMS->vBCFCPST = $vBCST;
                    $tagICMS->pFCPST   = $rowImpostoTrib["VALORADICIONALALIQFECP"];
                    $tagICMS->vFCPST   = $rowImpostoTrib["VALORADICIONALFECP"];
                    $this->valorAdicionalFCPST += $rowImpostoTrib["VRADICFECP"];
                }
            }
            else{
                $vBCST = '0.00';
                $pICMSST = '0.00';
                $vICMSST = '0.00';
            }
        } else {
            $vBCST = '0.00';
            $pICMSST = '0.00';
            $vICMSST = '0.00';
        }
        
        $tagICMS->item = $nItem;
        $tagICMS->orig = $orig;
        $tagICMS->CST = $CST;
        $tagICMS->modBC = $modBC;
        $tagICMS->pRedBC = $pRedBC;
        $tagICMS->vBC = $vBC;
        $tagICMS->pICMS = $pICMS;
        $tagICMS->vICMS = $vICMS;
        $tagICMS->vICMSDeson = $vICMSDeson;
        $tagICMS->motDesICMS = $motDesICMS;
        $tagICMS->modBCST = $modBCST;
        $tagICMS->pMVAST = $pMVAST;
        $tagICMS->pRedBCST = $pRedBCST;
        $tagICMS->vBCST = $vBCST;
        $tagICMS->pICMSST = $pICMSST;
        $tagICMS->vICMSST = $vICMSST;

        $this->nfe->tagICMS($tagICMS);
    }

    /**
     * Grupo do IPI 0 ou 1 [imposto] [O]
     * @param $rowIPI
     */
    private function criaTagIPI($nItem, $rowIPI)
    {
        $clEnq = "";
        $CNPJProd = "";
        $cSelo = "";
        $qSelo = "";
        $CST = '';
        $vBC = '';
        $pIPI = '';
        $vIPI = '';
        
        if(!$rowIPI["CDENQLEGALIPI"]) throw new \Exception("Código de enquadramento legal do IPI deve ser informado. Item: ". $nItem);
        $cEnq = $rowIPI["CDENQLEGALIPI"];

        if (in_array($rowIPI["CST"], array("00", "49", "50", "99"))) {
            //Grupo do IPITrib CST 00, 49, 50 e 99 0 ou 1 [IPI]
            $CST = $rowIPI["CST"];
            if ($rowIPI["CDFISCALIT"] === "1") {
                //BC e Percentagem de IPI 0 ou 1 [IPITrib]
                $vBC = $rowIPI["BASECALCULO"];
                $pIPI = number_format($this->util->decimalToFloat($rowIPI["ALIQUOTA"]), 2, ".", "");
                //Grupo do IPITrib CST 00, 49, 50 e 99 0 ou 1 [IPI]
                $vIPI = number_format($this->util->decimalToFloat($rowIPI["VALORIMPOSTO"]), 2, ".", "");
                $this->valorTotalIPI += $this->util->decimalToFloat($rowIPI["VRIMPOSTOIT"]);
            } else {
                //BC e Percentagem de IPI 0 ou 1 [IPITrib]
                $vBC = "0.00";
                $pIPI = "0.00";
                $vIPI = "0.00";
            }
        } elseif (in_array($rowIPI["CST"], array("01", "02", "03", "04", "05", "51", "52", "53", "54", "55"))) {
            //IPINT
            //Grupo IPI Não tributavel 0 ou 1 [IPI]
            $CST = $rowIPI["CST"];
        }
        
        $tagIPI = new \stdClass();
        
        $tagIPI->item = $nItem;
        $tagIPI->CST = $CST;
        $tagIPI->clEnq = $clEnq;
        $tagIPI->CNPJProd = $CNPJProd;
        $tagIPI->cSelo = $cSelo;
        $tagIPI->qSelo = $qSelo;
        $tagIPI->cEnq = $cEnq;
        $tagIPI->vBC = $vBC;
        $tagIPI->pIPI = $pIPI;
        $tagIPI->vIPI = $vIPI;
        
        
            
        $this->nfe->tagIPI($tagIPI);
    }

    /**
     * Grupo do Imposto de Importação 0 ou 1 [imposto] [P]
     * @param $rowII
     * @param $impostos
     */
    private function criaTagII($nItem, $rowII, $impostos)
    {
        $vII = 0;
        $vIOF = "";
        $vBC = $rowII["BASECALCULO"];
        if (!empty($rowII["VRDESPADUANEIRA"])) {
            $vDespAdu = number_format($this->util->decimalToFloat($rowII["DESPESASADUANEIRAS"]), 2, ".", "");
        } else {
            $vDespAdu = "0.00";
        }
        if ($rowII["CDFISCALIT"] === "1") {
            $vII = number_format($this->util->decimalToFloat($rowII["VALORIMPOSTO"]), 2, ".", "");
            $this->valorTotalVII += $this->util->decimalToFloat($rowII["VRIMPOSTOIT"]);
        }
        //IOF
        $aux = array_search("40", array_column($impostos, "CDIMPOSTRIB"));
        $IOF = $aux !== false ? $impostos[$aux] : '';
        if ($IOF) {
            if ($IOF["CDFISCALIT"] === "1") {
                $vIOF = number_format($this->util->decimalToFloat($IOF["VALORIMPOSTO"]), 2, ".", "");
            }
        } else {
            $vIOF = "0.00";
        }
        
        $tagII = new \stdClass();
        
        $tagII->item = $nItem;
        $tagII->vBC = $vBC;
        $tagII->vDespAdu = $vDespAdu;
        $tagII->vII = $vII;
        $tagII->vIOF = $vIOF;
        
        $this->nfe->tagII($tagII);
    }

    /**
     * Grupo do PIS obrigatorio [imposto] [Q]
     * @param $rowPIS
     * @param $ISSQN
     * @param $impostos
     * @return array
     */
    private function criaTagPIS($nItem, $rowPIS, $ISSQN = null, $impostos)
    {
        if ($rowPIS["CST"] === "01" || $rowPIS["CST"] === "02") {
            //PISAliq [Q02]
            //Grupo de PIS tributado pela alíquota 0 pou 1 [PIS]
            $CST = $rowPIS["CST"];
            if ($rowPIS["CDFISCALIT"] === "1") {
                $vBC = number_format($this->util->decimalToFloat($rowPIS["BASECALCULO"]), 2, ".", "");
                $pPIS = number_format($this->util->decimalToFloat($rowPIS["ALIQUOTA"]), 2, ".", "");
                $vPIS = number_format($this->util->decimalToFloat($rowPIS["VALORIMPOSTO"]), 2, ".", "");
                //PIS ISSQN
                if (isset($this->tags["ISSQN"])) {
                    $this->valorTotalPISISSQN += $this->util->decimalToFloat($rowPIS["VRIMPOSTOIT"]);
                } else {
                    $this->valorTotalPIS += $this->util->decimalToFloat($rowPIS["VRIMPOSTOIT"]);
                }
            } else {
                $vBC = "0.00";
                $pPIS = "0.00";
                $vPIS = "0.00";
            }
            
            $tagPIS = new \stdClass();
            $tagPIS->item = $nItem;
            $tagPIS->CST  = $CST;
            $tagPIS->vBC  = $vBC;
            $tagPIS->pPIS = $pPIS;
            $tagPIS->vPIS = $vPIS;
            
            $this->nfe->tagPIS($tagPIS);
            
        } elseif ($rowPIS["CST"] === "03") {
            //PISQtde [Q03]
            $CST = $rowPIS["CST"];
            $qBCProd = $rowPIS["QTITEMPROD"];
            if ($rowPIS["CDFISCALIT"] === "1") {
                $vAliqProd = number_format($this->util->decimalToFloat($rowPIS["VALORIMPOSTOALIQ"]), 4, ".", "");
                $vPIS = number_format($this->util->decimalToFloat($rowPIS["VALORIMPOSTO"]), 2, ".", "");
                //PIS ISSQN
                if ($ISSQN) {
                    $this->valorTotalPISISSQN += $this->util->decimalToFloat($rowPIS["VRIMPOSTOIT"]);
                } else {
                    $this->valorTotalPIS += $this->util->decimalToFloat($rowPIS["VRIMPOSTOIT"]);
                }
            } else {
                $vAliqProd = "0.0000";
                $vPIS = "0.00";
            }
            
            
            $tagPIS = new \stdClass();
            $tagPIS->item = $nItem;
            $tagPIS->CST  = $CST;
            $tagPIS->vPIS = $vPIS;
            $tagPIS->qBCProd    = $qBCProd;
            $tagPIS->vAliqProd  = $vAliqProd;
            
            $this->nfe->tagPIS($tagPIS);
            
        } elseif (in_array($rowPIS["CST"], array("04", "06", "07", "08", "09"))) {
            //PISNT [Q04]
            //Grupo de PIS não tributado 0 ou 1 [PIS]
            $CST = $rowPIS["CST"];
            
            $tagPIS = new \stdClass();
            $tagPIS->item = $nItem;
            $tagPIS->CST  = $CST;

            $this->nfe->tagPIS($tagPIS);
        } elseif (in_array($rowPIS["CST"], array("05", "49", "50", "51", "52", "53", "54", "55", "56", "60", "61", "62", "63",
                "64", "65", "66", "67", "70", "71", "72", "73", "74", "75", "98", "99"))
        ) {
            //PISOutr [Q05]
            $CST = $rowPIS["CST"];
            //                    $qBCProd = $rowPIS["QTITEMPROD"];

            if ($rowPIS["CDFISCALIT"] === "1") {
                //                        $vAliqProd = number_format($this->util->decimalToFloat($rowPIS["VALORIMPOSTOALIQ"]), 4);
                //Valor da Base de Cálculo do PIS e Alíquota do PIS (em percentual) 0 pu 1 [PISOutr]
                $vBC = number_format($this->util->decimalToFloat($rowPIS["BASECALCULO"]), 2, ".", "");
                $pPIS = number_format($this->util->decimalToFloat($rowPIS["ALIQUOTA"]), 2, ".", "");
                //Grupo de PIS Outras Operações 0 ou 1 [PIS]
                $vPIS = number_format($this->util->decimalToFloat($rowPIS["VALORIMPOSTO"]), 2, ".", "");
                //PIS ISSQN
                if ($ISSQN) {
                    $this->valorTotalPISISSQN += $this->util->decimalToFloat($rowPIS["VRIMPOSTOIT"]);
                } else {
                    $this->valorTotalPIS += $this->util->decimalToFloat($rowPIS["VRIMPOSTOIT"]);
                }
            } else {
                //                        $vAliqProd = "0.0000";
                //Valor da Base de Cálculo do PIS e Alíquota do PIS (em percentual) 0 pu 1 [PISOutr]
                $vBC = "0.00";
                $pPIS = "0.00";
                //Grupo de PIS Outras Operações 0 ou 1 [PIS]
                $vPIS = "0.00";
                //Quantidade Vendida e Alíquota do PIS (em reais) 0 ou 1 [PISOutr]
                //                    $this->txtStructure[] = "Q10|$qBCProd|$vAliqProd|";
            }
            
            $tagPIS = new \stdClass();
            
            $tagPIS->item = $nItem;
            $tagPIS->CST  = $CST;
            $tagPIS->vBC  = $vBC;
            $tagPIS->pPIS = $pPIS;
            $tagPIS->vPIS = $vPIS;
            
            $this->nfe->tagPIS($tagPIS);

            if (!empty($rowPIS["CDIMPOSTRIB"])) {
                $aux = array_search($rowPIS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTRIB"));
                $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
                if ($rowImpostrib) {
                    //Grupo de PIS Substituição Tributária 0 ou 1 [imposto]
                    $vPIS = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");

                    //Valor da Base de Cálculo do PIS e Alíquota do PIS (em percentual) 0 ou 1 [PISST]
                    $vBC = number_format($this->util->decimalToFloat($rowImpostrib["BASECALCULO"]), 2, ".", "");
                    $pPIS = number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");

                    $tagPISST = new \stdClass();
            
                    $tagPISST->item = $nItem;
                    $tagPISST->vBC  = $vBC;
                    $tagPISST->pPIS = $pPIS;
                    $tagPISST->vPIS = $vPIS;
                
                    $this->nfe->tagPISST($tagPISST);
                }
            }
        }
    }

    /**
     * //Grupo do COFINS obrigatório [imposto] [S]
     * @param $rowCOFINS
     * @param $rowItemProd
     * @param $impostos
     */
    private function criaTagCOFINS($nItem, $rowCOFINS,$rowItemProd, $impostos)
    {
        if ($rowCOFINS["CST"] === "01" || $rowCOFINS["CST"] === "02") {
            $this->criaTagCOFINSAliq($nItem, $rowCOFINS);
        } elseif ($rowCOFINS["CST"] === "03") {
            $this->criaTagCOFINSQtde($nItem, $rowCOFINS, $rowItemProd);
        } elseif (in_array($rowCOFINS["CST"], array("04", "06", "07", "08", "09"))) {
            $this->criaTagCOFINSNT($nItem, $rowCOFINS);
        } elseif (in_array($rowCOFINS["CST"], array("05", "49", "50", "51", "52", "53", "54", "55", "56", "60", "61", "62", "63", "64", "65",
                "66", "67", "70", "71", "72", "73", "74", "75", "98", "99"))) {
            $this->criaTagCOFINSOutr($nItem, $rowCOFINS);
            $this->criaTagCOFINSST($nItem, $rowCOFINS, $impostos);
        }
    }

    private function criaTagCOFINSAliq($nItem, $rowCOFINS)
    {
        //COFINSAliq
        //Grupo de COFINS tributado pela alíquota 0 ou 1 [COFINS]
        $CST = $rowCOFINS["CST"];
        if ($rowCOFINS["CDFISCALIT"] === "1") {
            $vBC = number_format($this->util->decimalToFloat($rowCOFINS["BASECALCULO"]), 2, ".", "");
            $pCOFINS = number_format($this->util->decimalToFloat($rowCOFINS["ALIQUOTA"]), 2, ".", "");
            $vCOFINS = number_format($this->util->decimalToFloat($rowCOFINS["VALORIMPOSTO"]), 2, ".", "");
            //COFINS ISSQN
            if (isset($this->tags["ISSQN"])) {
                $this->valorTotalCOFINSISSQN += $this->util->decimalToFloat($rowCOFINS["VRIMPOSTOIT"]);
            } else {
                $this->valorTotalCOFINS += $this->util->decimalToFloat($rowCOFINS["VRIMPOSTOIT"]);
            }
        } else {
            $vBC = "0.00";
            $pCOFINS = "0.00";
            $vCOFINS = "0.00";
        }
        
        $tagCOFINS = new \stdClass();
        
        $tagCOFINS->item = $nItem;
        $tagCOFINS->CST = $CST;
        $tagCOFINS->vBC = $vBC;
        $tagCOFINS->pCOFINS = $pCOFINS;
        $tagCOFINS->vCOFINS = $vCOFINS;
        
        $this->nfe->tagCOFINS($tagCOFINS);
    }

    /**
     * Grupo de COFINS tributado por Qtde 0 ou 1 [COFINS] [S03]
     * @param $rowCOFINS
     * @param $rowItemProd
     * @param $ISSQN
     */
    private function criaTagCOFINSQtde($nItem, $rowCOFINS, $rowItemProd)
    {
        //COFINSQtde
        $CST = $rowCOFINS["CST"];
        $qBCProd = $rowItemProd["QTITEMPROD"];
        if ($rowCOFINS["CDFISCALIT"] === "1") {
            $vAliqProd = number_format($this->util->decimalToFloat($rowCOFINS["VALORIMPOSTOALIQ"]), 4, ".", "");
            $pCOFINS = number_format($this->util->decimalToFloat($rowCOFINS["ALIQUOTA"]), 2, ".", "");
            $vCOFINS = number_format($this->util->decimalToFloat($rowCOFINS["VALORIMPOSTO"]), 2, ".", "");
            //COFINS ISSQN
            if (isset($this->tags["ISSQN"])) {
                $this->valorTotalCOFINSISSQN += $this->util->decimalToFloat($rowCOFINS["VRIMPOSTOIT"]);
            } else {
                $this->valorTotalCOFINS += $this->util->decimalToFloat($rowCOFINS["VRIMPOSTOIT"]);
            }
        } else {
            $vAliqProd = "0.0000";
            $pCOFINS = "0.00";
            $vCOFINS = "0.00";
        }
        
        $tagCOFINS = new \stdClass();
        
        $tagCOFINS->item = $nItem;
        $tagCOFINS->CST = $CST;
        $tagCOFINS->pCOFINS = $pCOFINS;
        $tagCOFINS->vCOFINS = $vCOFINS;
        $tagCOFINS->qBCProd = $qBCProd;
        $tagCOFINS->vAliqProd = $vAliqProd;
        
        $this->nfe->tagCOFINS($tagCOFINS);
    }

    /**
     * Grupo de COFINS não tributado 0 ou 1 [COFINS] [S04]
     * @param $rowCOFINS
     */
    private function criaTagCOFINSNT($nItem, $rowCOFINS)
    {
        //COFINSNT
        $CST = $rowCOFINS["CST"];
        
        $tagCOFINS = new \stdClass();
        
        $tagCOFINS->item = $nItem;
        $tagCOFINS->CST = $CST;

        $this->nfe->tagCOFINS($tagCOFINS);
    }

    /**
     * Grupo de COFINS Outras Operações 0 ou 1 [COFINS] [S05]
     * @param $rowCOFINS
     */
    private function criaTagCOFINSOutr($nItem, $rowCOFINS)
    { //COFINSOutr
        $CST = $rowCOFINS["CST"];
        //                    $qBCProd = $rowItemProd["QTITEMPROD"];
        if ($rowCOFINS["CDFISCALIT"] === "1") {
            //                        $vAliqProd = number_format($this->util->decimalToFloat($rowCOFINS["VALORIMPOSTOALIQ"]), 4);
            //Valor da Base de Cálculo da COFINS e Alíquota da COFINS (em percentual) 0 ou 1 [COFINSOutr]
            $vBC = number_format($this->util->decimalToFloat($rowCOFINS["BASECALCULO"]), 2, ".", "");
            $pCOFINS = number_format($this->util->decimalToFloat($rowCOFINS["ALIQUOTA"]), 2, ".", "");
            //Grupo de COFINS Outras Operações 0 ou 1 [COFINS]
            $vCOFINS = number_format($this->util->decimalToFloat($rowCOFINS["VALORIMPOSTO"]), 2, ".", "");
            //COFINS ISSQN
            if (isset($this->tags["ISSQN"])) {
                $this->valorTotalCOFINSISSQN += $this->util->decimalToFloat($rowCOFINS["VRIMPOSTOIT"]);
            } else {
                $this->valorTotalCOFINS += $this->util->decimalToFloat($rowCOFINS["VRIMPOSTOIT"]);
            }
        } else {
            //                        $vAliqProd = "0.0000";
            //Valor da Base de Cálculo da COFINS e Alíquota da COFINS (em percentual) 0 ou 1 [COFINSOutr]
            $vBC = "0.00";
            $pCOFINS = "0.00";
            //Grupo de COFINS Outras Operações 0 ou 1 [COFINS]
            $vCOFINS = "0.00";
        }
        
        $tagCOFINS = new \stdClass();
        
        $tagCOFINS->item = $nItem;
        $tagCOFINS->CST = $CST;
        $tagCOFINS->vBC = $vBC;
        $tagCOFINS->pCOFINS = $pCOFINS;
        $tagCOFINS->vCOFINS = $vCOFINS;
        
        $this->nfe->tagCOFINS($tagCOFINS);
    }

    /**
     * [T] & [T02]
     * @param $rowCOFINS
     * @param $impostos
     */
    private function criaTagCOFINSST($nItem, $rowCOFINS, $impostos)
    {
        //COFINSST
        if (!empty($rowCOFINS["CDIMPOSTRIB"])) {
            $aux = array_search($rowCOFINS["CDIMPOSTRIB"], array_column($impostos, "CDIMPOSTRIB"));
            $rowImpostrib = $aux !== false ? $impostos[$aux] : '';
            if ($rowImpostrib) {
                //                            $qBCProd = $rowImpostrib["QTITEMPROD"];
                //                            $vAliqProd = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTOALIQ"]), 4);
                //Grupo de COFINS Substituição Tributária 0 ou 1 [imposto] [T]
                $vCOFINS = number_format($this->util->decimalToFloat($rowImpostrib["VALORIMPOSTO"]), 2, ".", "");
                //Valor da Base de Cálculo da COFINS e Alíquota da COFINS (em percentual) 0 ou 1 [COFINSST] [T02]
                $vBC = number_format($this->util->decimalToFloat($rowImpostrib["BASECALCULO"]), 2, ".", "");
                $pCOFINS = number_format($this->util->decimalToFloat($rowImpostrib["ALIQUOTA"]), 2, ".", "");
                //Quantidade Vendida e Alíquota da COFINS (em reais) 0 u 1 [COFINSST]
                //                            $this->txtStructure[] = "T04|$qBCProd|$vAliqProd|";
                
                $tagCOFINSST = new \stdClass();
        
                $tagCOFINSST->item = $nItem;
                $tagCOFINSST->vBC = $vBC;
                $tagCOFINSST->pCOFINS = $pCOFINS;
                $tagCOFINSST->vCOFINS = $vCOFINS;
                
                $this->nfe->tagCOFINSST($tagCOFINSST);
            }
        }
    }

    /**
     * Grupo de Valores Totais da NF-e obrigatorio [infNFe]
     * @param $rowItemProd
     */
    private function criaTagDadosTotais()
    {
        $this->criaTagDadosTotaisICMS();
        if (isset($this->tags["ISSQN"])) {
            $this->criaTagDadosTotaisISSQN();
        }
    }

    /**
     * Grupo de Valores Totais referentes ao ICMS obrigatorio [total]
     * @param $rowItemProd
     */
    private function criaTagDadosTotaisICMS()
    {
        $vBC = number_format($this->valorTotalBC, 2, ".", "");
        $vICMS = number_format($this->valorTotalICMS, 2, ".", "");
        $vICMSDeson = number_format($this->valorTotalICMSDeson, 2, ".", "");
        $vBCST = number_format($this->valorTotalBCST, 2, ".", "");
        $vST = number_format($this->valorTotalICMSST, 2, ".", "");
        $vICMSUFDest = number_format($this->valorTotalPartDest, 2, ".", "");
        $vICMSUFRemet = number_format($this->valorTotalPartRemet, 2, ".", "");

        //Valor tota do itens sem desconto
        $vProd = number_format($this->valorTotalVPROD, 2, ".", "");

        // Inicia dataset dos encargos para funcoes de soma de valor encargo
        $this->encargosNota = $this->transmissaoNfeFactory->consultaEncargosNotaFiscal($this->corpoNF["NRLANCTONF"], $this->corpoNF["CDFILIAL"]);
 
        $somaDescontoItem = $this->util->decimalToFloat($this->transmissaoNfeFactory->getSomaDescontoItem($this->corpoNF["NRLANCTONF"], $this->corpoNF["CDFILIAL"]));
        // ENCARGO [01]
        $vFrete = number_format($this->somaValorEncargo("01"), 2, ".", "");
        // ENCARGO [02]
        $vSeg = number_format($this->somaValorEncargo("02"), 2, ".", "");
        // ENCARGO [03]
        $vFrete = number_format($this->somaValorEncargo("01"), 2, ".", "");
        // ENCARGO [02]
        $vSeg = number_format($this->somaValorEncargo("02"), 2, ".", "");
        // ENCARGO [03]
        if(sizeof($this->transmissaoNfeFactory->consultaDesoneraItem($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"], "T")) !== 0) {
            $rowDesonera = $this->transmissaoNfeFactory->consultaDesoneraItem($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"], "T")[0];
        } else {
            $rowDesonera = 0;
        } 
        //Valor total do itens sem desconto
        $dstTotalItensNF = $this->transmissaoNfeFactory->somaTotalItensNF($this->corpoNF["NRLANCTONF"], $this->corpoNF["CDFILIAL"]);

        if ($vFrete > 0 || $somaDescontoItem > 0 || !empty($dstTotalItensNF) || $this->util->decimalToFloat($rowDesonera["VRIMPOSTOIT"]) > 0) {
            $vDesc = number_format($this->somaValorEncargo("03") + $somaDescontoItem + $this->util->decimalToFloat($rowDesonera["VRIMPOSTOIT"]), 2, ".", "");
        } else {
            $vDesc = "0.00";
        }

        $vII = number_format($this->valorTotalVII, 2, ".", "");
        $vIPI = number_format($this->valorTotalIPI, 2, ".", "");
        $vPIS = number_format($this->valorTotalPIS, 2, ".", "");
        $vCOFINS = number_format($this->valorTotalCOFINS, 2, ".", "");

        // ENCARGO [04]
        $vOutro = number_format($this->somaValorEncargo("04"), 2, ".", "");
        // TOTAL DA NOTA FISCAL
        $vNF = number_format($this->util->decimalToFloat($this->corpoNF["VRNOTAFISC"]), 2, ".", "");
        
        $tagICMSTot = new \stdClass(); 
        $tagICMSTot->vBC = $vBC;
        $tagICMSTot->vICMS = $vICMS;
        $tagICMSTot->vICMSDeson = $vICMSDeson;
        $tagICMSTot->vBCST = $vBCST;
        $tagICMSTot->vST = $vST;
        $tagICMSTot->vProd = $vProd;
        $tagICMSTot->vFrete = $vFrete;
        $tagICMSTot->vSeg = $vSeg;
        $tagICMSTot->vDesc = $vDesc;
        $tagICMSTot->vII = $vII;
        $tagICMSTot->vIPI = $vIPI;
        $tagICMSTot->vPIS = $vPIS;
        $tagICMSTot->vCOFINS = $vCOFINS;
        $tagICMSTot->vOutro = $vOutro;
        $tagICMSTot->vNF = $vNF;
        $tagICMSTot->vICMSUFDest = $vICMSUFDest;
        $tagICMSTot->vICMSUFRemet = $vICMSUFRemet;
        
        // Versão 4.00
        $tagICMSTot->vFCP =  number_format($this->valorAdicionalFCP, 2, ".", "");
        $tagICMSTot->vFCPST =  number_format($this->valorAdicionalFCPST, 2, ".", "");
        $tagICMSTot->vFCPSTRet =  number_format($this->valorAdicionalFCPSTRet, 2, ".", "");
        
        $this->nfe->tagICMSTot($tagICMSTot);
        
        $this->valorTotalBC           = 0.00;
        $this->valorTotalICMS         = 0.00;
        $this->valorTotalICMSDeson    = 0.00;
        $this->valorTotalBCST         = 0.00;
        $this->valorTotalICMSST       = 0.00;
        $this->valorTotalIPI          = 0.00;
        $this->valorTotalCOFINS       = 0.00;
        $this->valorTotalVII          = 0.00;
        $this->valorTotalPIS          = 0.00;
        $this->valorTotalVPROD        = 0.00;
        $this->valorTotalPartDest     = 0.00;
        $this->valorTotalPartRemet    = 0.00;
    }

    /**
     * Grupo de Valores Totais referentes ao ISSQN 0 ou 1 [total]
     */
    private function criaTagDadosTotaisISSQN()
    {
        if ($this->valorTotalVSERV > 0) {
            $vServ = number_format($this->valorTotalVSERV, 2, ".", "");
            $vBC = "";
            if ($this->valorTotalVBCISSQN > 0)
                $vBC = number_format($this->valorTotalVBCISSQN, 2, ".", "");
            $vISS = "";
            if ($this->valorTotalVISS > 0)
                $vISS = number_format($this->valorTotalVISS, 2, ".", "");
            $vPIS = "";
            if ($this->valorTotalPISISSQN > 0)
                $vPIS = number_format($this->valorTotalPISISSQN, 2, ".", "");
            $vCOFINS = "";
            if ($this->valorTotalCOFINSISSQN > 0)
                $vCOFINS = number_format($this->valorTotalCOFINSISSQN, 2, ".", "");
            
            $tagISSQNTot = new \stdClass();
            
            $tagISSQNTot->vServ = $vServ;
            $tagISSQNTot->vBC = $vBC;
            $tagISSQNTot->vISS = $vISS;
            $tagISSQNTot->vPIS = $vPIS;
            $tagISSQNTot->vCOFINS = $vCOFINS;
            
            $this->nfe->tagISSQNTot($tagISSQNTot);
            
            $this->valorTotalVSERV        = 0.00;
            $this->valorTotalVBCISSQN     = 0.00;
            $this->valorTotalVISS         = 0.00;
            $this->valorTotalPISISSQN     = 0.00;
            $this->valorTotalCOFINSISSQN  = 0.00;
        }
    }

    //retTrib
    //$resp = $nfe->tagretTrib($vRetPIS, $vRetCOFINS, $vRetCSLL, $vBCIRRF, $vIRRF, $vBCRetPrev, $vRetPrev);

    /**
     * Grupo de Informações do Transporte da NF-e obrigatorio [infNFe] [X]
     */
    private function criaTagTransportadora()
    {
        //Dados transportadora
        if ($this->corpoNF["IDPGTOFRETE"] != '') {
            $modFrete = $this->corpoNF["IDPGTOFRETE"];
            
            $tagTransp  = new \stdClass();
            $tagTransp->modFrete = $modFrete;
            
            $this->nfe->tagtransp($tagTransp);

            if (!empty($this->corpoNF["NRINSJURTRAN"])) {
                $this->criaTagTransportador();
            }
        } else {
            $tagTransp  = new \stdClass();
            $tagTransp->modFrete = "9";

            $this->nfe->tagtransp($tagTransp);
        }
    }

    //valores retidos para transporte
    //$vServ = '258,69'; //Valor do Serviço
    //$vBCRet = '258,69'; //BC da Retenção do ICMS
    //$pICMSRet = '10,00'; //Alíquota da Retenção
    //$vICMSRet = '25,87'; //Valor do ICMS Retido
    //$CFOP = '5352';
    //$cMunFG = '3509502'; //Código do município de ocorrência do fato gerador do ICMS do transporte
    //$resp = $nfe->tagretTransp($vServ, $vBCRet, $pICMSRet, $vICMSRet, $CFOP, $cMunFG);

    /**
     * Grupo Transportador 0 ou 1 [transp] [X03]
     */
    private function criaTagTransportador()
    {
        $xNome = '';
        $IE = '';
        $xEnder = '';
        $xMun = '';
        $UF = '';
        $CPF = '';
        $CNPJ = '';

        if (!empty($this->corpoNF["NMRAZSOCTRAN"])) {
            $xNome = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMRAZSOCTRAN"])));
        }

        if (!empty($this->corpoNF["NRINSESTTRAN"])) {
            if (strlen($this->masks->removeMascaraInscricao($this->corpoNF["NRINSESTTRAN"])) >= 2 && strlen($this->masks->removeMascaraInscricao($this->corpoNF["NRINSESTTRAN"])) <= 14) {
                $IE = $this->masks->removeCharsFromString($this->masks->removeAcentos($this->corpoNF["NRINSESTTRAN"]));
            }
        }

        if (!empty($this->corpoNF["DSENDETRAN"])) {
            $xEnder = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["DSENDETRAN"])));
        }

        if (!empty($this->corpoNF["NMMUNITRAN"])) {
            $xMun = trim(strtoupper($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMMUNITRAN"]))));
        }

        if (!empty($this->corpoNF["SGESTATRAN"])) {
            $UF = trim(strtoupper($this->corpoNF["SGESTATRAN"]));
        }

        //CPF 0 ou 1 [transporta] [X05]
        if ($this->corpoNF["IDTPIJURTRAN"] === "F") {
            $CPF = $this->masks->removeMascaraInscricao($this->corpoNF["NRINSJURTRAN"]);
        } else {
            //CNPJ 0 ou 1 [transporta] [X04]
            $CNPJ = $this->masks->removeMascaraInscricao($this->corpoNF["NRINSJURTRAN"]);
            $CNPJ = str_pad(substr($CNPJ, strlen($CNPJ) - 14, 14), 14, "0");
        }

        $tagTransporta = new \stdClass();

        $tagTransporta->xNome = $xNome;
        $tagTransporta->IE = $IE;
        $tagTransporta->xEnder = $xEnder;
        $tagTransporta->xMun = $xMun;
        $tagTransporta->UF = $UF;
        $tagTransporta->CNPJ = $CNPJ;
        $tagTransporta->CPF  = $CPF;

        $this->nfe->tagtransporta($tagTransporta);
    }

    private function criaTagVeiculoTransportadora()
    {
        if (!empty($this->corpoNF["NRPLACAVEIC"])) {
            $placa = trim(strtoupper($this->corpoNF["NRPLACAVEIC"]));
        }

        if (!empty($this->corpoNF["SGUFVEIC"])) {
            $veicUF = trim(strtoupper($this->corpoNF["SGUFVEIC"]));
        }

        $esp = "";
        $marca = "";
        $nVol = "";
        $pesoL = "";
        $pesoB = "";
        $RNTC = "";
        if (!empty($this->corpoNF["DSQTDEVOLU"])) {
            $qVol = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["DSQTDEVOLU"])));
        }

        if (!empty($this->corpoNF["DSESPECVOLU"])) {
            $esp = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["DSESPECVOLU"])));
        }

        if (!empty($this->corpoNF["DSMARCAVOLU"])) {
            $marca = trim(strtoupper($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["DSMARCAVOLU"]))));
        }

        if (!empty($this->corpoNF["DSNUMEVOLU"])) {
            $nVol = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["DSNUMEVOLU"])));
        }

        if (!empty($this->corpoNF["VRPLIQUVOLU"])) {
            $pesoL = $this->corpoNF["VRPLIQUVOLU"];
        }

        if (!empty($this->corpoNF["VRPBRUTOVOLU"])) {
            $pesoB = $this->corpoNF["VRPBRUTOVOLU"];
        }

        if (isset($placa) && isset($veicUF)) {
            //Grupo Veículo 0 ou 1 [transp] [X18]
            $tagVeicTransp = new \stdClass();
            
            $tagVeicTransp->placa  = $placa;
            $tagVeicTransp->UF = $veicUF;
            $tagVeicTransp->RNTC   = $RNTC;
            
            $this->nfe->tagveicTransp($tagVeicTransp);
        }

        //Grupo Volumes 0 a N [transp] [X26]
        if (isset($qVol)) {
            
            $tagVol = new \stdClass();
            $tagVol->qVol = $qVol;
            $tagVol->esp = $esp;
            $tagVol->marca = $marca;
            $tagVol->nVol = $nVol;
            $tagVol->pesoL = $pesoL;
            $tagVol->pesoB = $pesoB;
            
            
            $this->nfe->tagvol($tagVol);
        }
    }

    //dados dos reboques
    //$aReboque = array(
    //    array('ZZQ9999', 'SP', '', '', ''),
    //    array('QZQ2323', 'SP', '', '', '')
    //);
    //foreach ($aReboque as $reb) {
    //    $placa = $reb[0];
    //    $UF = $reb[1];
    //    $RNTC = $reb[2];
    //    $vagao = $reb[3];
    //    $balsa = $reb[4];
    //    //$resp = $nfe->tagreboque($placa, $UF, $RNTC, $vagao, $balsa);
    //}

    /**
     * Grupo de Cobrança 0 ou 1 [infNFe] [Y]
     */
    private function criaTagCobranca()
    {
        $contarec = $this->transmissaoNfeFactory->consultaContarecerNFE($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
        $this->criaTagPag();
        if (!empty($contarec)) {
            $this->criaTagFat();
            foreach($contarec as $conta) {
                $this->criaTagDuplicata($conta);
            }
        }
    }
    
    private function criaTagPag() {
        $infoPagamento = $this->transmissaoNfeFactory->getInfoPagamento($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
        $pagamentos = array();

        if(empty($infoPagamento)) {
            $geraInfoPagamento = $this->transmissaoNfeFactory->getGeraInfoPagamento($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
            if(empty($geraInfoPagamento) 
                || $this->corpoNF["IDNFAJUSTE"] == 'S'
                || $this->transmissaoNfeFactory->verificaNotaDevo($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]) == 'S'
                || ($this->corpoNF["CDFILIRECE"] != null && $this->corpoNF["CDFILIEMIT"])) {
                $vrPagamento = 0;
                $tpFormaPaga  = '';
                $tpMeioPaga   = '90';
            }
            else {
                $geraInfoPagamento = $geraInfoPagamento[0];
                $vrPagamento = $geraInfoPagamento["VRORIG"];
                if(strtotime($geraInfoPagamento["DTORIGVEN"]) - strtotime($this->corpoNF["DTENTRSAID"]) == 0) {
                    $tpFormaPaga = '0';
                }
                else {
                    $tpFormaPaga = '1';
                }
                $tpMeioPaga = '15';
            }
            $this->transmissaoNfeFactory->insereInfoPagaNfe($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"], '001', $tpFormaPaga, $vrPagamento, $tpMeioPaga, $this->corpoNF["NRORG"]);
            $infoPagamento = array(
                array(
                    "IDTPMEIOPAGA"  => $tpMeioPaga,
                    "IDTPFORMAPAGA" => $tpFormaPaga,
                    "VALORPAGAMENTO" => $vrPagamento
                )
            );
            
        }

        if(!empty($infoPagamento)) {
            $tagPag = new \stdClass();
            
            $valorPagamento = 0;

            foreach($infoPagamento as $info) {
                $tagDetPag = new \stdClass();
                
                if(!empty($info["IDTPFORMAPAGA"])) {
                    if($info["IDTPFORMAPAGA"]) $tagDetPag->indPag = $info["IDTPFORMAPAGA"];
                }
                
                if(gettype($info["VALORPAGAMENTO"]) == "string") {
                    $info["VALORPAGAMENTO"] = (float) str_replace(",", ".", $info["VALORPAGAMENTO"]) ;
                }
                
                $tagDetPag->tPag = $info["IDTPMEIOPAGA"];
                $tagDetPag->vPag = number_format($info["VALORPAGAMENTO"], 2, ".", "");
                
                $valorPagamento += $info["VALORPAGAMENTO"];
                
                if(!empty($info["IDTPINTPAGACARD"])) {
                    
                    $tagDetPag->tpIntegra = $info["IDTPINTPAGACARD"];
                    
                    if(!empty($info["NRINSJURCREDCARD"]))   $tagDetPag->CNPJ  = $info["NRINSJURCREDCARD"];
                    if(!empty($info["IDTPBANDCARD"]))       $tagDetPag->tBand = $info["IDTPBANDCARD"];
                    if(!empty($info["NRAUTOOPERACARD"]))    $tagDetPag->cAut  = $info["NRAUTOOPERACARD"];
                }
                
                array_push($pagamentos, $tagDetPag);
            } 

            if($info["IDTPMEIOPAGA"] != '90' && $info["IDTPMEIOPAGA"] != '99') {
                $troco = $valorPagamento - $this->corpoNF["VRNOTAFISC"];
                if($troco >= 0){
                    $tagPag->vTroco = number_format(($valorPagamento - $this->corpoNF["VRNOTAFISC"]), 2, ".", "");
                }
                
            }
            
            $this->nfe->tagpag($tagPag);
            
            foreach($pagamentos as $detPag) {
                $this->nfe->tagdetPag($detPag);
            }
        }
        
        
    }
    //dados da fatura
    // $nFat = '000035342';
    // $vOrig = '1200.00';
    // $vDesc = '';
    // $vLiq = '1200.00';
    // $resp = $nfe->tagfat($nFat, $vOrig, $vDesc, $vLiq);

    /**
     * Grupo da Duplicata 0 a N [cobr] [Y07]
     * @param $rowContarec
     */
     
    private function criaTagFat() {
        $fat = new \stdClass();
        
        $fat->nFat = $this->getNumeroNotaFiscal();
        $fat->vOrig = number_format($this->corpoNF["VRNOTAFISC"], 2, '.', '');
        $fat->vDesc = number_format('0', 2, '.', '');
        $fat->vLiq  = number_format($this->corpoNF["VRNOTAFISC"], 2, '.', '');
        
        $this->nfe->tagfat($fat);
    }
     
    private function criaTagDuplicata($rowContarec)
    {
        $dVenc = '';
        $nDup = str_pad($rowContarec["NRSEQVEN"], 3, '0', STR_PAD_LEFT);
        if (!empty($rowContarec["DTORIGVEN"])) {
            $dVenc = \DateTime::createFromFormat('d/m/Y H:i:s', $rowContarec["DTORIGVEN"], new \DateTimeZone('America/Sao_Paulo'))->format("Y-m-d");
        }

        $vDup = $rowContarec["VRORIG"];

        $dup = new \stdClass();
        $dup->nDup  = $nDup;
        $dup->dVenc = $dVenc;
        $dup->vDup  = $vDup;

        $this->nfe->tagdup($dup);
    }
    
    /**
     * Grupo Informações do Intermediador da Transação [infNFe] [YB]
     */
    private function criaTagInformacoesIntermediador() {
        
        $indPres = $this->corpoNF['IDPRESENCIAL'];
        
        if($this->corpoNF['IDINTERMED'] == 1 && ($indPres == 1 || $indPres == 2 || $indPres == 3 || $indPres == 4 || $indPres == 9)) {
            
            $nt2020006 = $this->transmissaoNfeFactory->atualizaXMLNT2020006(); //Atualizar este bloco após 01/09/2021.
            $gravaXml = false;
            
                 if($nt2020006['ATUALIZAPROD'] && $this->nfeTools->config->tpAmb == 1)    $gravaXml = true;
            else if($nt2020006['ATUALIZAHOMOLOG'] && $this->nfeTools->config->tpAmb == 2) $gravaXml = true;
            if($gravaXml) {
                $infIntermed = new \stdClass();
                $infIntermed->CNPJ = trim($this->masks->removeMascaraInscricao($this->corpoNF['NRCNPJINTERMED']));
                $infIntermed->idCadIntTran = substr(trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF['NMCADINTERMED']))), 0, 60);
                
                $this->nfe->tagIntermed($infIntermed);
            }
        }
    }
    
    /**
     * Grupo de Informações Adicionais 0 ou 1 [infNFe] [Z]
     */
    private function criaTagInformacoesAdicionais()
    {
        $infAdFisco  = "";
        $infComplGeral = "";
        
        if (isset($this->info)) 
            $infAdFisco = trim(str_replace(chr(13), "", str_replace(chr(10), "", substr($this->info, 0, 2000))));
        
        $dadosadic = $this->transmissaoNfeFactory->getDadosAdic($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);

        $i = 0; 
        $dadosad = array();
        
        foreach($dadosadic as $dados){
            $dados = $dadosadic[$i]["DSDADOADIC"];
            $i++;
            array_push( $dadosad, $this->util->convertISOToUTF8(trim ($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->info . " " . $dados))) ));

        }
        
        //Rotina especifica do LabGlobo.
        $lista = $this->transmissaoNfeFactory->consultaListaNFE($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]);
        $infCompl = "";
        
        
        if (!empty($lista)){
            foreach($lista as $list) {
                $infCompl = trim($infCompl . " " . $this->masks->removeAcentos($list["DSLISTA"]));
            }
        }
        if(!empty($infCompl) || !empty($dadosad)) {
            $infCpl = "";
            
            // foreach($dadosad as $dados){
            //     $infCpl = implode$infCpl." | ",trim(str_replace(chr(13), "", str_replace(chr(10), "", substr($infCompl . $dados, 0, 5000))));
            // }
            
            $infCpl = trim(implode(' ; ', $dadosad));

            $infAdic = new \stdClass();
            
            $infAdic->infAdFisco = $infAdFisco;
            $infAdic->infCpl     = $infCpl;
            
            $this->nfe->taginfAdic($infAdic);

            // ESSA ROTINA DE FISCO DEVE SER ADAPTADA PARA ACEITAR MAIS DE UMA OCORRÊNCIA. POIS O PRÓPRIO MANUAL PERMITE ATÉ 10 OCORRÊNCIAS
            $objDadosfisco = Dadosfisco::getObjectByWhere("CDFILIAL = ? AND NRLANCTONF = ? AND TO_NUMBER(NRDADOFISCO) <= 10", array($this->corpoNF["CDFILIAL"], $this->corpoNF["NRLANCTONF"]));
            if ($objDadosfisco instanceof Dadosfisco) {
                $this->criaTagFisco($objDadosfisco);
            }

        }
        // Informacoes de comercio exterior
        if ($this->corpoNF["IDENTRSAIDOP"] === "S") {
            if ($this->corpoNF["CDPAISBACENCLIE"] !== "1058") {
                if($this->boGeraTagExporta){
                    $this->criaTagExportacao();
                }
            }
        }
    }
    
    private function criaTagInfoCompras(){
        $infoCompra =  $this->transmissaoNfeFactory->getInfoCompra($this->corpoNF['CDFILIAL'], $this->corpoNF['NRLANCTONF']);
        
        //verificando se existem informações para serem adicionadas
        if($infoCompra['DSNOTAEMPENHO'] || $infoCompra['CDPEDIDO'] || $infoCompra['CDCONTRATO']){
            
            $tagcompra = new \stdClass();
            $tagcompra->xNEmp = $infoCompra['DSNOTAEMPENHO'];
            $tagcompra->xPed =  $infoCompra['CDPEDIDO'];
            $tagcompra->xCont = $infoCompra['CDCONTRATO']; 
            
            $resp = $this->nfe->tagcompra($tagcompra);
        }
        
    }

    //observações emitente
    //$aObsC = array(
    //    array('email','roberto@x.com.br'),
    //    array('email','rodrigo@y.com.br'),
    //    array('email','rogerio@w.com.br'));
    //foreach ($aObsC as $obs) {
    //    $xCampo = $obs[0];
    //    $xTexto = $obs[1];
    //    $resp = $nfe->tagobsCont($xCampo, $xTexto);
    //}

    /**
     * Grupo do campo de uso livre do Fisco 0-10 [infAdic] [Z07]
     * @param $objDadosfisco
     */
    private function criaTagFisco(Dadosfisco $objDadosfisco)
    {
        $xCampo = $objDadosfisco->getCddadofisco();
        $xTexto = $objDadosfisco->getDsdadofisco();
        
        $tagObsFisco = new \stdClass();
        $tagObsFisco->xCampo = $xCampo;
        $tagObsFisco->xTexto = $xTexto;

        $this->nfe->tagobsFisco($tagObsFisco);
    }

    //Dados do processo
    //0=SEFAZ; 1=Justiça Federal; 2=Justiça Estadual; 3=Secex/RFB; 9=Outros
    //$aProcRef = array(
    //    array('nProc1','0'),
    //    array('nProc2','1'),
    //    array('nProc3','2'),
    //    array('nProc4','3'),
    //    array('nProc5','9')
    //);
    //foreach ($aProcRef as $proc) {
    //    $nProc = $proc[0];
    //    $indProc = $proc[1];
    //    //$resp = $nfe->tagprocRef($nProc, $indProc);
    //}

    /**
     * Grupo de Exportação 0 ou 1 [infNFe] [ZA]
     */
    private function criaTagExportacao()
    {
        $UFEmbarq = trim($this->corpoNF["SGESTAEMIT"]);
        $xLocExporta = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($this->corpoNF["NMMUNIEMIT"])));
        
        $tagExporta = new \stdClass();
        $tagExporta->UFSaidaPais = $UFEmbarq;
        $tagExporta->xLocExporta  = $xLocExporta;
        
        $this->nfe->tagexporta($tagExporta);
    }

    //dados de compras
    //$xNEmp = '';
    //$xPed = '12345';
    //$xCont = 'A342212';
    //$resp = $nfe->tagcompra($xNEmp, $xPed, $xCont);

    //dados da colheita de cana
    //$safra = '2014';
    //$ref = '01/2014';
    //$resp = $nfe->tagcana($safra, $ref);
    //$aForDia = array(
    //    array('1', '100', '1400', '1000', '1400'),
    //    array('2', '100', '1400', '1000', '1400'),
    //    array('3', '100', '1400', '1000', '1400'),
    //    array('4', '100', '1400', '1000', '1400'),
    //    array('5', '100', '1400', '1000', '1400'),
    //    array('6', '100', '1400', '1000', '1400'),
    //    array('7', '100', '1400', '1000', '1400'),
    //    array('8', '100', '1400', '1000', '1400'),
    //    array('9', '100', '1400', '1000', '1400'),
    //    array('10', '100', '1400', '1000', '1400'),
    //    array('11', '100', '1400', '1000', '1400'),
    //    array('12', '100', '1400', '1000', '1400'),
    //    array('13', '100', '1400', '1000', '1400'),
    ///    array('14', '100', '1400', '1000', '1400')
    //);
    //foreach ($aForDia as $forDia) {
    //    $dia = $forDia[0];
    //    $qtde = $forDia[1];
    //    $qTotMes = $forDia[2];
    //    $qTotAnt = $forDia[3];
    //    $qTotGer = $forDia[4];
    //    //$resp = $nfe->tagforDia($dia, $qtde, $qTotMes, $qTotAnt, $qTotGer);
    //}

    private function getFormaPagamento($identrsaidop, $cdfilial, $nrlanctonf, $idorigdestnf, $dtentrsaid)
    {
        if($idorigdestnf === "L"){
            return 0;
        }else{
            if($identrsaidop === "S"){
                $resultSetFormpgto = $this->transmissaoNfeFactory->consultaFormaPagamentoSaida($cdfilial, $nrlanctonf, $idorigdestnf, $dtentrsaid);
                if(empty($resultSetFormpgto)){
                    return 2;
                }else{
                    return $resultSetFormpgto[0]["IDFORMPGTSAP"];
                }
            }else{
                $resultSetFormpgto = $this->transmissaoNfeFactory->consultaFormaPagamentoEntrada($cdfilial, $nrlanctonf, $idorigdestnf, $dtentrsaid);
                if(empty($resultSetFormpgto)){
                    return 2;
                }else{
                    return $resultSetFormpgto[0]["IDFORMPGTSAP"];
                }
            }
        }
    }

    private function geraNumeroNotaFiscal()
    {
        if(empty($this->corpoNF["NRNOTAFISCALELE"])){
            $id = "NOTAFISCALELE" . $this->corpoNF["CDEMPRESA"] .
            preg_replace("/\s+/", "",$this->masks->removeMascaraInscricao($this->corpoNF["NRINSJUREMIT"])) .
            $this->corpoNF['CDSERIE'] . $this->nfeTools->config->tpAmb;

            $number = $this->novoCodigo->getCodeByTableName($id, $this->corpoNF["NRORG"], 1, 9);
        }else{
            $number = $this->corpoNF["NRNOTAFISCALELE"];
        }

        return $number;
    }

    private function somaValorEncargo($codigo)
    {
        $valorEncargo = 0.00;
        $aux = array_search($codigo, array_column($this->encargosNota, "CDENCARGO"));
        $encargo = $aux !== false ? $this->encargosNota[$aux] : '';
        if ($encargo) {
        //     foreach($encargo as $enc) {
                // $valorEncargo += $this->util->decimalToFloat($enc["VALORENCARGOFLOAT"]);
            $valorEncargo += $this->util->decimalToFloat($encargo["VALORENCARGOFLOAT"]);
        //     }
        }

        return (float)number_format($valorEncargo, 2, ".", "");
    }
    
}